function moms=computing_moments(N,basis_type,domain_type,polygon_sides,...
    spline_order_vett,SPLtypestring,rotation,...
    P,Q,cubature_type,cumulative,a_tpz,b_tpz,g1,g2,a,b,c,d)


%--------------------------------------------------------------------------
% THE MAIN PURPOSE OF THIS ROUTINE IS TO COMPUTE MOMENTS OF A CERTAIN BASIS
% DEFINED BY basis_type AND COMPUTED IN vandermonde_2D, ON A CERTAIN DOMAIN
% DETERMINED BY domain_type AND DEFINED IN define_polygon. 
%
% DUE TO THE GENERAL FORM OF DOMAINS, THERE ARE SEVERAL INPUTS TO DEFINE 
% THEM, AS  spline_order_vett,SPLtypestring,cumulative FOR A CURVILINEAR 
% DOMAIN, a_tpz,b_tpz,g1,g2 FOR A TRAPEZOIDAL DOMAIN.
%
% THE PARAMETERS a,b,c,d ARE NEEDED TO POSSIBLY SCALE SOME BASIS AS 
% MONOMIAL, LEGENDRE OR CHEBYSHEV.
%
% FINALLY P, Q, rotation ARE PARAMETERS THAT DETERMINE A CERTAIN ROTATION 
% OF THE DOMAIN AND THE AXIS OF ROTATION. cubature_type DETERMINE THE
% CUBATURE USED BY SPLINEGAUSS. IMPORTANT CASES ARE "0" (PADUA POINTS
% IN SQUARE PANELS, ONLY FOR CURVILINEAR POLYGONS!), WHILE "4" IS USED FOR 
% TENSORIAL GAUSS-LEGENDRE (IN ANY SQUARE PANEL).
%--------------------------------------------------------------------------
% INPUTS.
%----------
%
% N : DEGREE OF THE VANDERMONDE BASIS.
%
% basis_type: IT DETERMINES THE BASIS FOR THE DOMAIN.
%
%    [basis_type = 0] MONOMIAL (NOT SCALED).
%    [basis_type = 1] LEGENDRE (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                     CONTAINING THE DOMAIN.
%    [basis_type = 2] CHEBYSHEV (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                     CONTAINING THE DOMAIN.
%    [basis_type = 3] MONOMIAL (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                     CONTAINING THE DOMAIN.
%    [basis_type = 4] ORTHONORMAL DUBINER BASIS FOR UNIT TRIANGLE 
%                     [0 0; 1 0; 0 1].
%    [basis_type = 5] LOGAN SHEPP BASIS FOR UNIT DISC B(C=(0,0), r=1).
%    [basis_type = 6] SCALED MONOMIAL BASIS FOR THE INTERVAL.
%    [basis_type = 7] SCALED LEGENDRE BASIS FOR THE INTERVAL.
%    [basis_type = 8] SCALED CHEBYSHEV BASIS FOR THE INTERVAL.
%    [basis_type = 9] KOORNWINDER BASIS FOR UNIT DISC B(C=(0,0), r=1).
%    [basis_type = 10] DUBINER ORTHOGONAL BASIS FOR UNIT SIMPLEX W.R.T.
%                      THE EQUILIBRIUM MEASURE.
%    [basis_type =11] FAST, SCALED CHEBYSHEV VANDERMONDE.

% domain_type: IT DETERMINES THE DOMAIN AS DESCRIBED IN THE M-FILE
%              "define_polygon.m". SOME EXAMPLES ARE:
%
%    [polygon_type=0]: UNIT DISK (parametrized as disc).
%    [polygon_type=1]: UNIT SQUARE [0,1]^2.
%    [polygon_type=2]: A 6 SIDES CONVEX POLYGON USED IN PAPER [1].
%    [polygon_type=3]: A 9 SIDES NONCONVEX POLYGON USED IN THE PAPER [1].
%    [polygon_type=4]: A 9 SIDES POLYGON.
%    [polygon_type=5]: A 10 SIDES POLYGON.
%    [polygon_type=6]: UNIT SQUARE [-1,1]^2.
%    [polygon_type=7]: UNIT TRIANGLE [0 0; 1 0; 0 1; 0 0].
%    [polygon_type=8]: ECLYPSE.
%    [polygon_type=9]: POINTS ON MONITOR.
%    [polygon_type=10]: CIRCLE (APPROXIMATED BY SPLINE POLYGON).
%    [polygon_type=11]: BOOMERANG
%    [polygon_type=12]: QUADRANGLE
%    [polygon_type=13]: NON CONVEX POLYGON, HORSE LIKE
%    [polygon_type=14]: NON CONVEX POLYGON, HAND'); 
%    [polygon_type=15]: UNIT TRIANGLE [0 0; 1 0; 1 1; 0 0].
%    [polygon_type=16]: RANDOM TRIANGLE.
%    [polygon_type=17]: AN HEXAGON.
%    [polygon_type=15]: INTERVAL [-1,1].
%
%    TO DEFINE YOUR OWN POLYGON, PLEASE OPEN THE m-FILE "define_polygon.m" 
%    AND ADD IT TO THE LIST. OBSERVE THAT THE FIRST VERTEX IS NOT REPEATED. 
%    FOR EXAMPLE, THE UNIT SQUARE IS DEFINED AS "[0 0; 1 0; 1 1; 0 1]" 
%    AND NOT "[0 0; 1 0; 1 1; 0 1; 0 0]".
%
% polygon_sides : THE DEFINITION OF THE DOMAIN COMES FROM DETERMINATING
%    SOME SAMPLINGS OF THE DOMAIN.
%    MORE PRECISELY  IF THE POLYGON HAS "L" SIDES, "polygon_sides"
%    IS A VARIABLE CONTAINING ITS VERTICES, ORDERED COUNTERCLOCKWISE.
%    AS LAST ROW MUST HAVE THE COMPONENTS OF THE FIRST VERTEX.
%     
%    IN OTHER WORDS, THE FIRST ROW AND LAST ROW ARE EQUAL.
%                   
%    "polygon_sides" IS A "L+1 x 2" MATRIX.
%                  IN THIS DEMO, THE POLYGON IS DEFINED BY THE ROUTINE
%                  "define_polygon.m".
% 
% spline_order_vett:SPLINE ORDER AND BLOCK (VECTOR).
%
%    [spline_order_vett(i,1)=2] : PIECEWISE LINEAR SPLINES.
%    [spline_order_vett(i,1)=4] : CUBIC SPLINES
%                                (DEPENDING ON "spline_type" IN INPUT).
%    [spline_order_vett(i,1)=k] : IN THE CASE k IS NOT 2 OR 4 IT
%                                 CHOOSES THE k-TH ORDER SPLINES
%                                 (FOR ADDITIONAL HELP DIGIT "help spapi"
%                                 IN MATLAB SHELL).
%
%    [spline_order_vett(:,2)]  : VECTOR OF FINAL COMPONENTS OF A BLOCK.
%
%    EXAMPLE:
%
%    "spline_order_vett=[2 31; 4 47; 8 67]" MEANS THAT FROM THE 1st
%    VERTEX TO THE 31th VERTEX WE HAVE AN ORDER 2 SPLINE (piecewise
%    linear), FROM THE 32th VERTEX TO THE 47th WE USE A 4th ORDER
%    SPLINE (i.e. A CUBIC AND PERIODIC SPLINE BY DEFAULT), FROM THE
%    48th TO THE 67th (AND FINAL!) WE USE AN 8th ORDER SPLINE.
%
% SPLtypestring: IF THE MATRIX (M X 2) "spline_order_vett" HAS SOME ELEMENT
%   IN THE FIRST COLUMN EQUAL TO ONE, SPLtypestring IT DECIDES THE TYPE OF 
%   END CONDITIONS OF THE CUBIC SPLINE IN THE PERTINENT BLOCK. 
%   IT IS A STRING. IT CAN BE:
%
%   complete'   : match endslopes (as given in VALCONDS,
%                 with default as under *default*).
%   'not-a-knot' : make spline C^3 across first and last
%                  interior break (ignoring VALCONDS if given).
%   'periodic'   : match first and second derivatives at
%                 first data point with those at last data
%                 point (ignoring VALCONDS if given).
%   'second'     : match end second derivatives (as given
%                  in VALCONDS,with default [0 0], i.e.,
%                 as in variational).
%   'variational': set end second derivatives equal to zero
%                  (ignoring VALCONDS if given).

% rotation: SUPPOSE x_min, x_max, y_min, y_max ARE THE MINIMUM AND MAXIMUM
%    VALUES IN x AND y REACHED BY THE POLYGON, I.E.
%    "R=[x_min,x_max] x [y_min,y_max]" IS THE SMALLEST RECTANGLE WITH
%    SIDES PARALLEL TO THE AXIS x AND y, CONTAINING THE POLYGON.
%
%   [rotation=0]: GENERAL CASE, BUT OBSERVE THAT THE FUNCTION MUST BE DEFINED
%                 IN THE RECTANGLE "R" DESCRIBED ABOVE.
%   [rotation=1]: GOOD FOR CONVEX POLYGONS. FOR CONVEX POLYGONS IT SUFFICES
%                 THAT THE FUNCTION IS DEFINED IN THE POLYGON.
%   [rotation=2]: THE USER CHOOSES A SPECIAL REFERENCE SEGMENT "PQ".
%                 CHECK [1] FOR FURTHER DETAILS. SEE THE VARIABLES "P", "Q"
%                 BELOW.
%
% P,Q: IF "rotation=2" THEN THE ALGORITHM CHOOSES A PREFERRED SEGMENT
%            "PQ" HAVING "P", "Q" AS EXTREMA, AS SET BY THE USER.
%            "P" AND "Q" ARE "1 x 2" ROW VECTORS. (SEE [3] FOR DETAILS).
% 
% cubature_type: IT CHOOSES THE QUADRATURE RULE USED BY SPLINEGAUSS.
%
%   [cubature_type=0]: PADUA POINTS OVER SQUARE PANELS.
%   [cubature_type=1]: TENSORIAL FEJER 1.
%   [cubature_type=2]: TENSORIAL FEJER 2.
%   [cubature_type=3]: TENSORIAL CLENSHAW CURTIS (VIA WEIGHTS).
%   [cubature_type=4]: TENSORIAL GAUSS-LEGENDRE.
%   [cubature_type=5]: TENSORIAL GAUSS-LEGENDRE-LOBATTO.

% cumulative: IT CHOOSES THE PARAMETRIZATION OF THE BOUNDARY WHEN IS 
%   DEFINED PARAMETRICALLY VIA SPLINES (SEE [3] FOR DETAILS).
%
%   [cumulative=0]: 1:N.
%   [cumulative=1]: CUMULATIVE.
%
% a_tpz,b_tpz, g1,g2 : IF [polygon_type=-1] THE TRAPEZOIDAL REGION IS 
%  DESCRIBED BY A SMALLER VALUED FUNCTION "g1" AND A BIGGER ONE "g2",
%  AND THE "x" INTERVAL [a_trap,b_trap].
%
%   EXAMPLES:
%   1. a_tpz=-1; b_tpz=1; g1=inline('x'); g2=inline('x+3');
%   2. a_tpz=-1; b_tpz=1; g1=inline('-ones(size(x))');
%      g2=inline('x/4+3/4');
%   3. a_tpz=-1; b_tpz=1; g1=inline('x'); g2=inline('x+3');
% 
% a, b, c, d: THEY DETERMINE THE SMALLER SQUARE INCLUDING THE DOMAIN (AT
%    LEAST NUMERICALLY) WITH SIDES PARALLEL TO THE CARTESIAN AXIS. IT IS  
%    USED TO SCALE SOME BASIS. 
% 
%----------
% OUTPUT:
%----------
%
% moms: A COLUMN VECTOR WITH THE MOMENTS OF THE BASIS ON THE DOMAIN.
%--------------------------------------------------------------------------
% REFERENCE PAPERS.
%-------------------
%
% [1] L. BOS and N. LEVENBERG
%     "On the Approximate Calculation of Fekete Points: the Univariate Case"
%     ETNA (to appear)
%
% [2] J.P. CALVI and N. LEVENBERG
%     "Uniform approximation by discrete least squares polynomials"
%     J. Approx. Theory 152 (2008), 82-100.
%
% [3] A. SOMMARIVA and M. VIANELLO
%     "Gauss-Green cubature over spline curvilinear polygons". Preprint.
%
% [4] A. SOMMARIVA and M. VIANELLO
%     "Computing approximate Fekete points by QR factorizations
%     of Vandermonde matrices".
%     Comp. Math. Appl. (to appear)
%--------------------------------------------------------------------------
% FUNCTIONS USED IN THIS ROUTINE.
%---------------------------------
% 
% 1. cubature_trapezoidal_region
% 2. vandermonde_polygons
% 3. cubature_disk
% 4. koornwinder
% 5. splinegauss_2009
% 6. quadrature_rules_1D
% 7. vandermonde_1D
%
% ALL THESE FUNCTIONS ARE EXTERNAL TO THIS ROUTINE.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 12, 2009
%--------------------------------------------------------------------------

switch domain_type
    
% COMPUTING MOMENTS ON A TRAPEZOIDAL REGION.
case -1
    
    [pts,w]=cubature_trapezoidal_region(N+5,cubature_type,a_tpz,b_tpz,g1,g2);
    
    nodes_x=pts(:,1);
    nodes_y=pts(:,2);
   
    V=vandermonde_polygons(basis_type,[nodes_x nodes_y],N,a,b,c,d);
    
    moms=V'*w;
    
    
% COMPUTING MOMENTS ON UNIT DISK.
case 0
    
    moms=[];
    
    
switch basis_type 
case 5
    m=N+1;
    s=(m^2+m)/2-1;
    moms=[sqrt(pi); zeros(s,1)];
case 9
    [nodes_x,nodes_y,weights]=cubature_disk(100, cubature_type);
    V=koornwinder(N,nodes_x,nodes_y);
    moms=V'*weights;
otherwise
    fprintf('\n \t [WARNING]: BASIS ON THE DISK NOT AVAILABLE.');    
end


% COMPUTING MOMENTS ON SYMPLEX (UNIT TRIANGLE).
case 7
    
moms=[];
switch basis_type 
    
case 4
    s=((N+2)*(N+1))/2-1;
    moms=[sqrt(2)/2; zeros(s,1)]; 
case 10
    [nodes_x,nodes_y,weights]=splinegauss_2009(N,polygon_sides,rotation,...
        P,Q,spline_order_vett,cumulative,SPLtypestring,cubature_type);
    V=dubiner_simplex_equilibrium(N,nodes_x,nodes_y);
    moms=V'*weights;
otherwise
    [nodes_x,nodes_y,weights]=splinegauss_2009(N,polygon_sides,rotation,...
        P,Q,spline_order_vett,cumulative,SPLtypestring,cubature_type);
    V=vandermonde_polygons(basis_type,[nodes_x nodes_y],N,a,b,c,d);
    moms=V'*weights;
end


% COMPUTING MOMENTS ON A BOUNDED INTERVAL (a,b)
case 18
    
    [nodes_x,w]=quadrature_rules_1D(N+10,4);
    nodes_x=((a+b)/2)+((-a+b)/2)*nodes_x;
    w=(b-a)*w/2;

    switch basis_type
        case 8
            V=vandermonde_1D(2,nodes_x,N,a,b); % CHEBYSHEV (IN (a,b)).
        case 7
            V=vandermonde_1D(1,nodes_x,N,a,b); % LEGENDRE (IN (a,b)).
        case 6
            V=vandermonde_1D(3,nodes_x,N,a,b); % MONOMIAL (IN (a,b)).
    end

    moms=V'*w;

% COMPUTING MOMENTS ON GENERAL CURVILINEAR SPLINE POLYGON.
otherwise
    
    [nodes_x,nodes_y,weights]=splinegauss_2009(N,polygon_sides,rotation,...
        P,Q,spline_order_vett,cumulative,SPLtypestring,cubature_type);
    % V=[phi_1(:) phi_2(:) ...].
    
    V=vandermonde_polygons(basis_type,[nodes_x nodes_y],N,a,b,c,d);
    moms=V'*weights;
    
end


