
function [nodes_x,nodes_y,weights]=cubature_disk(N, cub_type, angle_min,...
    angle_max)

%--------------------------------------------------------------------------
% OBJECT:
%----------
%
% COMPUTATION OF INTEGRAL ON A DISK (SECTOR) BY A QUADRATURE RULE WITH "N"
% NODES ON THE RADIAL PART AND ON THE ANGLE COMPONENT.
%
% PLEASE NOTICE THAT ONE COULD USE A COMPOSITE TRAPEZOIDAL RULE ON THE 
% ANGLE COMPONENT. 
%
%--------------------------------------------------------------------------
% INPUTS:
%---------- 
%
% N   : NUMBER OF POINTS OF A QUADRATURE RULE (THAT MAY HAVE ALGEBRAIC 
%       DEGREE OF PRECISION HIGHER THAT "N" (AS IN THE CASE OF GAUSS
%       LEGENDRE RULE).
% 
% cub_type : IT CHOOSES THE 1D QUADRATURE RULE USED BY cubature_disk.
%
%       [cubature_type=1]: FEJER 1.
%       [cubature_type=2]: FEJER 2.
%       [cubature_type=3]: CLENSHAW CURTIS (VIA WEIGHTS).
%       [cubature_type=4]: GAUSS-LEGENDRE.
%       [cubature_type=5]: GAUSS-LEGENDRE-LOBATTO.
%       [cubature_type=6]: COMPOSITE TRAPEZOIDAL RULE.
%
% angle_min, angle_max: MINIMUM AND MAXIMUM ANGLE OF THE SECTOR.
%
%--------------------------------------------------------------------------
% OUTPUTS:
%---------- 
%
% nodes_x,nodes_y: NODES (x,y) OF A CUBATURE RULE OVER THE DISK.
%
% weights: : WEIGHTS OF A CUBATURE RULE OVER THE DISK WITH NODES (x,y).
%
%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES :
%----------------------
%
% 1. quadrature_rules_1D
%
% THE ROUTINE quadrature_rules_1D.m IS NOT ATTACHED TO THIS FILE. 
%
%--------------------------------------------------------------------------
% EXAMPLE.
%----------
% [nodes_x,nodes_y,weights]=cubature_disk(3, 4);
% [nodes_x, nodes_y, weights]
% 
% ans =
% 
%-3.471592210148684e-02     6.012974091137855e-02     2.529212826992110e-02
%-3.471592210148688e-02    -6.012974091137854e-02     2.529212826992110e-02
% 6.943184420297371e-02    -1.700589715241390e-17     2.529212826992110e-02
%-1.650047391037858e-01     2.857965916174043e-01     2.253716599380364e-01
%-1.650047391037860e-01    -2.857965916174042e-01     2.253716599380364e-01
% 3.300094782075718e-01    -8.082901023503820e-17     2.253716599380364e-01
%-3.349952608962139e-01     5.802288121670344e-01     4.575531492587478e-01
%-3.349952608962143e-01    -5.802288121670343e-01     4.575531492587478e-01
% 6.699905217924281e-01    -1.641003495944324e-16     4.575531492587478e-01
%-4.652840778985129e-01     8.058956628730602e-01     3.389806137298925e-01
%-4.652840778985136e-01    -8.058956628730600e-01     3.389806137298925e-01
% 9.305681557970263e-01    -2.279234626770568e-16     3.389806137298925e-01
%
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 17, 2009
%--------------------------------------------------------------------------

if (nargin < 4)
    angle_min=0; angle_max=2*pi;
end

[r,w_r]=quadrature_rules_1D(N,cub_type); % 1D CUB RULE (SCALED IN [0,1])
r=0.5*(r+1);
w_r=real(w_r)*0.5;

h=(angle_max-angle_min)/N; 

if (angle_min == 0) & (angle_max == 2*pi)
    theta=h:h:2*pi; theta=theta';
else
    theta=angle_min:h:angle_max; theta=theta';
end

length_r=length(r);
length_theta=length(theta);

r_rep=repmat(r',length_theta,1); r_rep=r_rep(:);
theta_rep=repmat(theta,length_r,1);

nodes_x=r_rep.*cos(theta_rep);
nodes_y=r_rep.*sin(theta_rep);

w_theta=h*ones(size(theta));

w_r_rep=repmat(w_r',length_theta,1); w_r_rep=w_r_rep(:);
w_theta_rep=repmat(w_theta,length_r,1); 

weights=w_r_rep.*w_theta_rep.*r_rep;

