
function [pts,w]=cubature_trapezoidal_region(N,cubature_type,a,b,f1,f2)


%--------------------------------------------------------------------------
% OBJECT:
%-----------
% THE TARGET OF THIS ROUTINE IS TO PROVIDE "N^2" NODES "pts" AND WEIGHTS
% "w" OF A CUBATURE ROUTINE ON TRAPEZOIDAL REGIONS WHERE a <= x <= b AND
% f1(x) <= y <= f2(x).
%
%--------------------------------------------------------------------------
% INPUTS:
%----------
%
% N : NUMBER OF NODES OF 1D QUADRATURE RULE.
%
% cubature_type: IT POINTS A CUBATURE RULE
%
%   [cubature_type=1]: FEJER 1.
%   [cubature_type=2]: FEJER 2.
%   [cubature_type=3]: CLENSHAW CURTIS (VIA WEIGHTS).
%   [cubature_type=4]: GAUSS-LEGENDRE.
%   [cubature_type=5]: GAUSS-LEGENDRE-LOBATTO.
%
% a,b,f1,f2: THE TRAPEZOIDAL REGION IS DETERMINED BY THE COUPLES (X,Y) SUCH
%    THAT a <= x <= b AND f1(x) <= y <= f2(x).
%
%----------
% OUTPUTS:
%----------
%
% nodes : M x 2 MATRIX OF NODES, IN THE INTERVAL [-1,1].
%
% weights: M x 1 COLUMN VECTOR OF WEIGHTS.
%
%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES :
%----------------------
%
% 1. quadrature_rules_1D
%
% THIS ROUTINE IS NOT ATTACHED TO THIS FILE.
%
%--------------------------------------------------------------------------
% EXAMPLE:
%----------
%
% >> a=-1; b=1;
% >> cubature_type=4;
% >> N=2;
% >> g1=inline('-ones(size(x))'); g2=inline('x.^3/3-x.^2/12+7/12');
% >>[pts,w]=cubature_trapezoidal_region(2,cubature_type,a_trap,b_trap,g1,g2)
%
% pts =
%
%   -0.7746   -0.8447
%   -0.7746   -0.3108
%   -0.7746    0.2231
%         0   -0.8216
%         0   -0.2083
%         0    0.4049
%    0.7746   -0.8097
%    0.7746   -0.1559
%    0.7746    0.4980
%
%
% w =
%
%    0.2127
%    0.3403
%    0.2127
%    0.3909
%    0.6255
%    0.3909
%    0.2605
%    0.4169
%    0.2605
%
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 13, 2009
%--------------------------------------------------------------------------

method_tz=1;

if (method_tz == 1)
    
    % METHOD WITH MESHGRIDS.
    
    g1 = fcnchk(f1);
    g2 = fcnchk(f2);

    [nodes,weights]=quadrature_rules_1D(N,cubature_type);
    nodes=(nodes+1)/2;
    weights=weights/2;
    
    [X,Y]=meshgrid(nodes);
    
    g1X=feval(g1,X);
    g2X=feval(g2,X);
    Y=g1X+(g2X-g1X).*Y;

    pts=[X(:) Y(:)];
   
    [Wx,Wy]=meshgrid(weights);
    WW=(b-a)*Wx.*Wy.*(g2X-g1X);
    w=WW(:);

else

    % METHOD WITH REPMATS.
    
    g1 = fcnchk(f1);
    g2 = fcnchk(f2);

    [nodes,weights]=quadrature_rules_1D(N,cubature_type);

    s=repmat(nodes',length(nodes),1);
    t=repmat(nodes,1,length(nodes));

    ws=repmat(weights',length(weights),1);
    wt=repmat(weights,1,length(weights));
    wst=ws.*wt;

    hs=(b*(1+s)/2)+(a*(1-s)/2);

    g1_hs=feval(g1,hs);
    g2_hs=feval(g2,hs);

    hts=g2_hs.*(1+t)/2 + g1_hs.*(1-t)/2;
    jacb=((b-a)/2)*(g2_hs-g1_hs)/2;

    w=wst.*jacb; w=w(:);
    pts=[hs(:) hts(:)];
    
end



