
%--------------------------------------------------------------------------
% THE FOLLOWING CODE IMPLEMENTS AN ALGORITHM FOR COMPUTING APPROXIMATE
% FEKETE POINTS FOR MODERATE DEGREES, ON (SOME) REGIONS INCLUDING THOSE 
% WHOSE BOUNDARY CAN BE WELL REPRESENTED BY CURVILINEAR SPLINES.
%
%                        ...... IMPORTANT ......
% THE PROGRAM IS NOT SUITABLE FOR OCTAVE BECAUSE OF THE PARTICULAR 
% IMPLEMENTATION OF MATLAB ROUTINE "backslash".
%
%                          ...... USAGE ......
%
% 1. IF YOU INTEND TO ADD YOUR OWN WAM, ADD IT TO THE ROUTINE "wam.m".
%    SET A RIGHT CASE IN "switch wam_select" COMMAND AT THE BEGINNING OF 
%    THE ROUTINE "wam.m".
%
% 2. IF YOU INTEND TO ADD A FUNCTION, ADD IT TO "fct2D.m". IN PARTICULAR 
%    ADD A NEW CASE TO "switch function_type" IN "fct2D.m". 
%
% 3. IF YOU INTEND TO ADD A NEW POLYNOMIAL BASIS, SUITABLE FOR YOUR DOMAIN,
%    ADD IT TO "vandermonde2D.m". IN PARTICULAR ADD A NEW CASE TO "switch
%    basis_type" IN "vandermonde2D.m".
%
%--------------------------------------------------------------------------
% REFERENCE PAPERS.
%-------------------
%
% [1] L. BOS and N. LEVENBERG
%     "On the Approximate Calculation of Fekete Points: the Univariate Case"
%     ETNA.
%
% [2] L. BOS, N. LEVENBERG, J.P. CALVI, A. SOMMARIVA and M. VIANELLO
%     "Geometric Weakly Admissible Meshes, Discrete Least Squares 
%      Approximations and Approximate Fekete Points". Math. Comp. (to
%      appear).
%
% [3] L.BOS, A. SOMMARIVA, M. VIANELLO. "Least-squares polynomial 
%         approximation on weakly admissible meshes". Preprint.
%
% [4] J.P. CALVI and N. LEVENBERG
%     "Uniform approximation by discrete least squares polynomials".
%     J. Approx. Theory 152 (2008), 82-100.
%
% [5] A. SOMMARIVA and M. VIANELLO
%     "Gauss-Green cubature over spline curvilinear polygons". 
%     J. Comp. Appl. Math.
%
% [6] A. SOMMARIVA and M. VIANELLO
%     "Computing approximate Fekete points by QR factorizations
%     of Vandermonde matrices". Comp. Math. Appl. 
%
%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES :
%----------------------
%
% 1. define_domain
% 2. main_fekete
%
% THESE ROUTINES ARE EXTERNAL TO THIS DEMO FILE.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%%
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 17, 2009
%--------------------------------------------------------------------------


warning off; 
clear all; %clf;

%--------------------------------------------------------------------------
% extraction_method: 1: QR 2:LU.
%--------------------------------------------------------------------------
extraction_method=2;

%--------------------------------------------------------------------------
% SEVERAL POLYGONS ARE DEFINED IN THE m-FILE "define_domain.m".
% [polygon_type=-1]: TRAPEZOIDAL REGION (SEE g1, g2 BELOW). WORK IN
%                   PROGRESS.
% [polygon_type=0]: UNIT DISK.
% [polygon_type=1]: UNIT SQUARE [0,1]^2.
% [polygon_type=2]: A 6 SIDES CONVEX POLYGON USED IN PAPER [3].
% [polygon_type=3]: A 9 SIDES NONCONVEX POLYGON USED IN THE PAPER [3].
% [polygon_type=4]: A 9 SIDES POLYGON.
% [polygon_type=5]: A 10 SIDES POLYGON.
% [polygon_type=6]: UNIT SQUARE [-1,1]^2.
% [polygon_type=7]: SIMPLEX, i.e. UNIT TRIANGLE [0 0; 1 0; 0 1; 0 0].
% [polygon_type=8]: ECLYPSE.
% [polygon_type=9]: POINTS ON MONITOR.
% [polygon_type=10]: CIRCLE (APPROXIMATED BY SPLINE POLYGON).
% [polygon_type=11]: BOOMERANG
% [polygon_type=12]: QUADRANGLE
% [polygon_type=13]: NON CONVEX POLYGON.
% [polygon_type=14]: NON CONVEX POLYGON, HAND;
% [polygon_type=15]: UNIT TRIANGLE [0 0; 1 0; 1 1; 0 0].
% [polygon_type=16]: RANDOM TRIANGLE.
% [polygon_type=17]: NON REGULAR CONVEX HEXAGON (2009).
% [polygon_type=18]: INTERVAL [-1,1].
%
% TO DEFINE YOUR OWN POLYGON, PLEASE OPEN THE m-FILE "define_domain.m"
% AND ADD IT TO THE LIST. OBSERVE THAT THE FIRST VERTEX IS REPEATED.
% FOR EXAMPLE, THE UNIT SQUARE IS DEFINED AS "[0 0; 1 0; 1 1; 0 1 0 0]"
% AND NOT "[0 0; 1 0; 1 1; 0 1]".
%--------------------------------------------------------------------------
polygon_type=7;

%--------------------------------------------------------------------------
% [M] : APPROXIMATE FEKETE POINTS DEGREE.
%--------------------------------------------------------------------------
M=10;

%--------------------------------------------------------------------------
% [wam_select] -7: Interval. 1/N^2 equispaced points.
% [wam_select] -6: Simplex. New mesh v.2.0. (Oct. 2009).
% [wam_select] -5: Admissible mesh from tens. grid for disc (and find).
% [wam_select] -4: Points for a trapezoidal region (tensorial pts based). 
%                  Not a WAM, in general.
% [wam_select] -3: Admissible mesh from tens. grid (and inpolygon).
% [wam_select] -2: Points for simplex (2008).
% [wam_select] -1: Points for a trapezoidal region (Padua pts based). Not a
%                  WAM, in general.
% [wam_select]  0: Points for a disk (2008-2009).
% [wam_select]  1: Padua type panels over curvilinear polygons (via 
%                  splinegauss_2009).
% [wam_select]  2: Gaussian or Clenshaw Curtis or Fejer type
%                  over curvilinear polygons (via splinegauss_2009).
% [wam_select]  3: Minimal triangulation WAM (pts of a reference rectangle
%                  are mapped in any triangle of the triangulation).
% [wam_select]  4: Minimal triangulation WAM (pts of a reference triangle
%                  WAM are mapped in any triangle of the triangulation).
% [wam_select]  5: Minimal triangulation AM (pts of a reference triangle
%                  AM are mapped in any triangle of the triangulation).
% [wam_select]  6: Barycentric triangulation WAM (pts of a reference
%                  triangle WAM are mapped in any triangle of the
%                  triangulation). Good for star shaped or covex domains.
%--------------------------------------------------------------------------
wam_select=-6;

%--------------------------------------------------------------------------
% IF [polygon_type=-1] THE TRAPEZOIDAL REGION IS DESCRIBED BY A SMALLER
% VALUED FUNCTION "g1" AND A BIGGER ONE "g2", AND THE "x" INTERVAL
% [a_trap,b_trap].
%
% EXAMPLES:
% 1. a_trap=-1; b_trap=1; g1=inline('x'); g2=inline('x+3');
% 2. a_trap=-1; b_trap=1; g1=inline('-ones(size(x))');g2=inline('x/4+3/4');
% 3. a_trap=-1; b_trap=1; g1=inline('x'); g2=inline('x+3');
%--------------------------------------------------------------------------
a_trap=-1; b_trap=1;
g1=inline('-ones(size(x))'); g2=inline('x.^3/3-x.^2/12+7/12');
% g1=inline('-ones(size(x))'); g2=inline('(x+3)/4');

%--------------------------------------------------------------------------
% [spline_order_vett]: SPLINE ORDER AND BLOCK (VECTOR).
%
%        [spline_order_vett(i,1)=2] : PIECEWISE LINEAR SPLINES.
%        [spline_order_vett(i,1)=4] : CUBIC SPLINES
%                                    (DEPENDING ON "spline_type" IN INPUT).
%        [spline_order_vett(i,1)=k] : IN THE CASE k IS NOT 2 OR 4 IT
%                                     CHOOSES THE k-TH ORDER SPLINES
%                                    (FOR ADDITIONAL HELP DIGIT "help spapi"
%                                     IN MATLAB SHELL).
%
%        [spline_order_vett(:,2)]  : VECTOR OF FINAL COMPONENTS OF A BLOCK.
%
%        EXAMPLE:
%
%        "spline_order_vett=[2 31; 4 47; 8 67]" MEANS THAT FROM THE 1st
%        VERTEX TO THE 31th VERTEX WE HAVE AN ORDER 2 SPLINE (piecewise
%        linear), FROM THE 32th VERTEX TO THE 47th WE USE A 4th ORDER
%        SPLINE (i.e. A CUBIC AND PERIODIC SPLINE BY DEFAULT), FROM THE
%        48th TO THE 67th (AND FINAL!) WE USE AN 8th ORDER SPLINE.
%--------------------------------------------------------------------------
% [polygon_sides]: IF THE POLYGON HAS "L" SIDES, "polygon_sides" IS A 
%                  VARIABLE CONTAINING ITS VERTICES, ORDERED
%                  COUNTERCLOCKWISE. AS LASTROW MUST HAVE THE COMPONENTS 
%                  OF THE FIRST VERTEX.
%                  IN OTHER WORDS, THE FIRST ROW AND LAST ROW ARE EQUAL.
%                  "polygon_sides" IS A "L+1 x 2" MATRIX.
%                  IN THIS DEMO, THE POLYGON IS DEFINED BY THE ROUTINE
%                  "define_domain.m".
%
%                  NOTE: IT IS NOT A PARAMETER TO SET.
%--------------------------------------------------------------------------
polygon_sides=define_domain(polygon_type,g1,g2,a_trap,b_trap);

%**************************************************************************
% IMPORTANT: IF YOU DO NOT USE SPLINES OF ORDER DIFFERENT THAN 2, SET
%            spline_order_vett(1)=2.
%            EXAMPLE: IF YOU WORK ON A DISK, PUT "spline_order_vett(1)=2".
% spline_order_vett=[2 3; 4 size(polygon_sides,1)];
%**************************************************************************
spline_order_vett=[2 size(polygon_sides,1)];

%--------------------------------------------------------------------------
% [rotation]: SUPPOSE x_min, x_max, y_min, y_max ARE THE MINIMUM AND MAXIMUM
%            VALUES IN x AND y REACHED BY THE POLYGON, I.E.
%            "R=[x_min,x_max] x [y_min,y_max]" IS THE SMALLEST RECTANGLE WITH
%            SIDES PARALLEL TO THE AXIS x AND y, CONTAINING THE POLYGON.
%
%   [rotation=0]: GENERAL CASE, BUT OBSERVE THAT THE FUNCTION MUST BE DEFINED
%                 IN THE RECTANGLE "R" DESCRIBED ABOVE.
%   [rotation=1]: GOOD FOR CONVEX POLYGONS. FOR CONVEX POLYGONS IT SUFFICES
%                 THAT THE FUNCTION IS DEFINED IN THE POLYGON.
%   [rotation=2]: THE USER CHOOSES A SPECIAL REFERENCE SEGMENT "PQ".
%                 CHECK [1] FOR FURTHER DETAILS. SEE THE VARIABLES "P", "Q"
%                 BELOW.
%--------------------------------------------------------------------------
rotation=0;

%--------------------------------------------------------------------------
% [P, Q]   : IF [rotation=2] THEN THE ALGORITHM CHOOSES A PREFERRED SEGMENT
%            "PQ" HAVING "P", "Q" AS EXTREMA, AS SET BY THE USER.
%            "P" AND "Q" ARE "1 x 2" ROW VECTORS. (SEE [1] FOR DETAILS).
%--------------------------------------------------------------------------
P=[0 0]; Q=[1 0];

%--------------------------------------------------------------------------
% [cubature_type]: IT CHOOSES THE 1D QUADRATURE RULE USED BY splinegauss TO
%           OBTAIN A CUBATURE FORMULA ON A (CURVILINEAR) POLYGON.
%
%           [cubature_type=1]: FEJER 1.
%           [cubature_type=2]: FEJER 2.
%           [cubature_type=3]: CLENSHAW CURTIS (VIA WEIGHTS).
%           [cubature_type=4]: GAUSS-LEGENDRE.
%           [cubature_type=5]: GAUSS-LEGENDRE-LOBATTO.
%--------------------------------------------------------------------------
cubature_type=5;

%--------------------------------------------------------------------------
% THIS MATLAB CODE PERFORMS SEVERAL PURPOSES. IN THIS PART YOU CAN SELECT
% OUTPUTS ONLY FOR THOSE IN WHICH YOU ARE INTERESTED.
%
% 0: NO 1: YES.
%
% [output_parms(1) ]: LEBESGUE CONSTANT.
% [output_parms(2) ]: VANDERMONDE CONDITIONING.
% [output_parms(3) ]: IT PLOTS THE DOMAIN AND THE W.A.M. NODES.
% [output_parms(4) ]: VANDERMONDE DETERMINANTS.
% [output_parms(5) ]: INTERPOLATION STATS.
% [output_parms(6) ]: CUBATURE STATS.
%
% EXAMPLE.
%
% output_parms = [0 0 0 0 1 1];
%
% GIVES THE VALUES OF VANDERMONDE CONDITIONING, IT PLOTS THE DOMAIN AND
% THE W.A.M. NODES, IT GIVES SOME VANDERMONDE DETERMINANTS, INTERPOLATION &
% CUBATURE STATISTICS.
%--------------------------------------------------------------------------
output_parms = [0 1 1 1 1 1]; 

%--------------------------------------------------------------------------
% [basis_type = 0] MONOMIAL (NOT SCALED).
% [basis_type = 1] LEGENDRE (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                  CONTAINING THE DOMAIN.
% [basis_type = 2] CHEBYSHEV (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                  CONTAINING THE DOMAIN.
% [basis_type = 3] MONOMIAL (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                  CONTAINING THE DOMAIN.
% [basis_type = 4] ORTHONORMAL DUBINER BASIS FOR UNIT TRIANGLE (SIMPLEX)
%                  [0 0; 1 0; 0 1].
% [basis_type = 5] LOGAN SHEPP BASIS FOR UNIT DISC B(C=(0,0), r=1).
% [basis_type = 6] SCALED MONOMIAL BASIS FOR THE INTERVAL.
% [basis_type = 7] SCALED LEGENDRE BASIS FOR THE INTERVAL.
% [basis_type = 8] SCALED CHEBYSHEV BASIS FOR THE INTERVAL.
% [basis_type = 9] KOORNWINDER BASIS FOR UNIT DISC B(C=(0,0), r=1).
% [basis_type =10] DUBINER ORTHOGONAL BASIS FOR UNIT SIMPLEX W.R.T.
%                  THE EQUILIBRIUM MEASURE.
% [basis_type =11] FAST, SCALED CHEBYSHEV VANDERMONDE.
%--------------------------------------------------------------------------
% [niter]          NUMBER OF TIMES THAT THE DISCRETE ORTHOGONALIZATION
%                  PROCESS TAKES PLACE.
%                  [niter=0]: NO DISCRETE ORTHOGONALIZATION.
%--------------------------------------------------------------------------
basis_type=[10];
niter=[2];

%--------------------------------------------------------------------------
% [function_type]: TEST FUNCTION TO BE APPROXIMATED.
%
%                1: .75*exp(-((9*x-2).^2 + (9*y-2).^2)/4) + ...
%                    .75*exp(-((9*x+1).^2)/49 - (9*y+1)/10) + ...
%                    .5*exp(-((9*x-7).^2 + (9*y-3).^2)/4) - ...
%                    .2*exp(-(9*x-4).^2 - (9*y-7).^2);
%
%                2: ( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
%                3: (x+y).^20;
%                4: exp((x-0.5).^2+(y-0.5).^2);
%                5: exp(-100*((x-0.5).^2+(y-0.5).^2));
%                6: cos(30*(x+y));
%                7: ones(size(x));
%                8: exp(x+y);
%                9: 1./[1+16*(x.^2+y.^2)];
%                10: (x.^2+y.^2).^(3/2);
%                11: exp(-x.^2-y.^2);
%                12: exp(-[ (x.^2+y.^2) ]^(-1))
%                13: cos(x.^2+y.^2);
%                14: cos((x+y));
%                15: ((x-0.4)^2 +(y-0.4)^2 )^(1/2);
%--------------------------------------------------------------------------
function_type=[8];

%--------------------------------------------------------------------------
% [cumulative]: IT CHOOSES THE PARAMETRIZATION OF THE BOUNDARY.
%              [cumulative=0]: 1:N.
%              [cumulative=1]: CUMULATIVE.
%--------------------------------------------------------------------------
cumulative=0;

%--------------------------------------------------------------------------
% [SPLtypestring]: IF [spline_order_vett=4] IT DECIDES THE TYPE OF END
%                  CONDITIONS OF THE CUBIC SPLINE. IT IS A STRING.
%
%                  IT CAN BE:
%
%                 'complete'   : match endslopes (as given in VALCONDS,
%                                with default as under *default*).
%                 'not-a-knot' : make spline C^3 across first and last
%                              interior break (ignoring VALCONDS if given).
%                 'periodic'   : match first and second derivatives at
%                                first data point with those at last data
%                                point (ignoring VALCONDS if given).
%                 'second'     : match end second derivatives (as given
%                                in VALCONDS,with default [0 0], i.e.,
%                                as in variational).
%                 'variational': set end second derivatives equal to zero
%                                (ignoring VALCONDS if given).
%--------------------------------------------------------------------------
SPLtypestring='periodic';

%--------------------------------------------------------------------------
% [make_diary]: DIARY.
%              [make_diary=0]: NO DIARY.
%              [make_diary=1]: DIARY.
%--------------------------------------------------------------------------
make_diary=0;

%--------------------------------------------------------------------------
%                         DEMO STARTS HERE
%--------------------------------------------------------------------------

if make_diary == 1
    diary on; s=clock; ss=datenum(s); sss=datestr(ss,30); diary(sss);
end

[nodes_fek_x,nodes_fek_y,nodes_fek_cub_x,nodes_fek_cub_y,w_fek_cub,leb1,...
    ls_norm]=main_fekete(M,basis_type,polygon_type,polygon_sides,...
    spline_order_vett,SPLtypestring,rotation,P,Q,cubature_type,...
    cumulative,wam_select,niter,function_type,output_parms,g1,g2,...
    a_trap,b_trap,extraction_method);

if make_diary == 1
    diary off;
end

fprintf('\n \n');
