function V=dubiner_legendre_weight(pts,polynomial_degree)

%--------------------------------------------------------------------------
% INPUT.
%---------
%
% pts               : SET OF POINTS OF THE UNIT TRIANGLE [0 0; 1 0; 1 1] IN  
%                     WHICH THE POLYNOMIALS BASIS MUST BE EVALUATED.
%                     THEY MUST BE STORED IN A 2 COLUMS MATRIX.
%
% polynomial_degree : DEGREE OF THE VANDERMONDE MATRIX (wrt ORTHONORMAL 
%                     DUBINER BASIS, WRT THE LEGENDRE BIVARIATE WEIGHT).
%
%--------------------------------------------------------------------------
% OUTPUT:
%---------
%
% V  : VANDERMONDE MATRIX "V(pts,polynomial_degree)", WRT THE ORTHONORMAL
%    DUBINER BASIS AND EVALUATED IN THE POINTS THAT ARE THE COMPONENTS OF  
%    THE MATRIX pts. IF {\phi_k} IS A BASIS, THEN 
%
%         V(:)=[\phi_1(:) \phi_2(:) ... \phi_N(:)] 
%   
%    WHERE [\phi_k]_k IS THE ORTHONORMAL DUBINER BASIS.
%
%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES.
%----------------------
%
% 1. jac_polyval_orth_multiple
%
% THE ROUTINE IS BELOW THE CODE OF "dubiner_legendre_weight".
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>       
%% Date: November 17, 2009
%--------------------------------------------------------------------------


xi=pts(:,1);
eta=pts(:,2);

% Duffy's tranformation.
num_xi_prime=2*xi;
den_xi_prime=(1-abs(sign(1-eta)))*eps+(1-eta);
xi_prime=(num_xi_prime./den_xi_prime)-1;
eta_prime=2*eta-1;

% Legendre orthonormal polynomial evaluated on xi_prime.
legendre_mat=jac_polyval_orth_multiple(xi_prime,polynomial_degree,0,0);

V=[];
for k=0:polynomial_degree
    legendre_term=legendre_mat(:,k+1).* (1-eta).^k;
    jacobi_k_term=jac_polyval_orth_multiple(eta_prime,polynomial_degree-k,2*k+1,0);
    legendre_term_rep=repmat(legendre_term,1,polynomial_degree-k+1);
    % Jacobi orthonormal polynomials 
    % (k,l):=(k,0:polynomial_degree-k) 
    % evaluated on eta_prime.
    fx_rep=(sqrt(8)*(2^k))*legendre_term_rep  .* jacobi_k_term;
    V=[V fx_rep];
end


%-------------------------------
% jac_polyval_orth_multiple.
%-------------------------------

function y=jac_polyval_orth_multiple(x,n,a,b)

%--------------------------------------------------------------------------
% INPUTS.
%----------
%
% x: column vector of points in the interval [-1,1].
%
% n,a,b: degree of ORTHONORMAL Jacobi polynomial on [-1,1],
%        wrt. the weight w(x)=(1-x)^a * (1+x)^b
%
%--------------------------------------------------------------------------
% OUTPUT.
%---------
%
% y: evaluation in x of the n degree ORTHONORMAL Jacobi polynomial on [-1,1]
%    wrt. the weight w(x)=(1-x)^a * (1+x)^b.
%             
%--------------------------------------------------------------------------
% NOTE.
%--------
% GAUTSCHI ROUTINE r_jacobi COMPUTES RECURRENCE COEFF. OF MONIC JACOBI 
% POLYNOMIALS. 
%
% IN WOLFRAM HOMEPAGE:,
%
% http://mathworld.wolfram.com/JacobiPolynomial.html
%
% IT IS CONSIDERED ANOTHER FAMILY OF JACOBI POLYNOMIALS
% IN WHICH IT IS NORMALISED THE VALUE P_n(1).
% SINCE IN THE LATTER ONE KNOWS THE VALUE OF int(P^2_n,-1,1), WE USE THIS 
% RESULT TO COMPUTE ORTHONORMAL JACOBI POLYNOMIALS FOR "ALLOWED" INTEGER 
% a,b (SEE nchoosek, DOES NOT WORK WITH REAL VALUES OF a,b).
%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES :
%----------------------
%
% 1. r_jacobi
%
% THE ROUTINE quadrature_rules_1D.m IS NOT ATTACHED TO THIS FILE. IT IS
% SUPPLIED BY Dirk Laurie AND EDITED BY Walter Gautschi. IT CAN BE FOUND AT
% W. GAUTSCHI HOMEPAGE.
%
%--------------------------------------------------------------------------

if size(x,1) > size(x,2)
    col_var=1;
else
    col_var=0;
    x=x';
end

% PS: LAST COORDINATE OF x_EXT IS USED FOR NORMALISATION.
x_ext=[x; 1];

ab=r_jacobi(n+5,a,b);

y_ext=[zeros(size(x_ext,1),1) ones(size(x_ext,1),1)];

for index=1:n
    
    y_ext_loc=(x_ext-ab(index,1)).* y_ext(:,end) - ab(index,2) * y_ext(:,end-1);
    y_ext=[y_ext y_ext_loc];
    
end

y_ext=y_ext(:,2:end);
y=y_ext(1:end-1,:); 

n_vett=(0:n)';

% Evaluating normalisation (so to achieve orthonormal Jacobi polynomials).
phi_G_1=y_ext(end,:); phi_G_1=phi_G_1';

phi_N_1_vett=[];
for index=1:length(n_vett)
   n_loc=n_vett(index);
   phi_N_1_vett_loc=nchoosek(n_loc+a,n_loc);
   phi_N_1_vett=[phi_N_1_vett; phi_N_1_vett_loc];
end
c_1=phi_G_1./phi_N_1_vett;



num1=2^(a+b+1); den1=2*n_vett+a+b+1;
num2=gamma(n_vett+a+1); den2=gamma(n_vett+1);
num3=gamma(n_vett+b+1); den3=gamma(n_vett+a+b+1);

s_vett=sqrt((num1./den1).*(num2./den2).*(num3./den3));

scaling_mat=diag(1./(s_vett.*c_1));

y=y*scaling_mat;

