
function V=dubiner_simplex_equilibrium(M,x,y)

%--------------------------------------------------------------------------
% OBJECT.
%----------
%
% This function computes for fixed M,x,y, and all (m,n) 
% such that m=0,...,M, 0<=m+n<=M
%
% S_{m,n}(x,y)=[P_m(-1/2,-1/2)(x/(1-y))] * [(1-y)^m] * [P_n(2m,-1/2)(y)]
%
% that is the (m,n) Dubiner bivariate polynomial on the simplex of degree
% at most M, such that {S_{m,n}} forms an orthogonal system w.r.t. the 
% equilibrium measure on the simplex (P_n is the n degree monic Jacobi 
% orthogonal polynomial, in [0,1] and not in [-1,1]), i.e.
%
%                         mu(x,y)=x*y*(1-x-y)^{-1/2}
%
% All the values S_{m,n}(x,y) are stored in the matrix V, where 
% S_{m,n}(x,y) is a column vector.
% Particular care has been taken in the evaluation by 3 terms recursion of
%
%                    Q_{m}(x,y)={P_m(-1/2,-1/2)(x/(1-y))} * [(1-y)^m] 
%
% If we set a=(1-y), then Q_n(x)={a^n}*P_n(x/a), x in [0,1],  
% (alpha_n, beta_n) being the recursion terms of three terms recurrance 
% relation of the monic polynomial P_n, we easily have 
%
%                 Q_{n+1}(x) = (x-a1n)*Q_n(x)-b1n* Q_{n-1}(x)
%
% where a1n=a*alpha_n, b1n={a^2}*Q_{n-1}(x), Q_{-1}(x)=0, Q_{0}(x)=1.
%
% We observe that, if y=1 then by the two militia men theorem Q_{m}(x,y)=0.
%
%--------------------------------------------------------------------------
% INPUTS:
%---------
%
% M: DEGREE OF THE VANDERMONDE BASIS.
%
% x,y : POINTS (x,y) IN WHICH ALL THE POLYNOMIALS OF THE BASIS MUST BE 
%       EVALUATED.
% 
%---------- 
% OUTPUTS:
%----------
%
% V: VANDERMONDE MATRIX V(pts,polynomial_degree), WRT THE BASIS DETERMINED 
%    BY basis_type, EVALUATED IN THE POINTS THAT ARE THE COMPONENTS OF THE 
%    MATRIX pts. IF {\phi_k} IS A BASIS, THEN 
%
%               V(:)=[\phi_1(:) \phi_2(:) ... \phi_N(:)]
%
%--------------------------------------------------------------------------
% FUNCTIONS USED BY THIS ROUTINE.
%---------------------------------
% 
% 1. r_jacobi
% 2. scaled_orthogonal_polynomials
% 3. eval_orthogonal_polynomials
%
% THE FUNCTION r_jacobi IS EXTERNAL TO THIS ROUTINE. r_jacobi IS
% WRITTEN BY D. LAURIE AND EDITED BY W. GAUTSCHI. 
%
% THE FUNCTIONS scaled_orthogonal_polynomials, eval_orthogonal_polynomials
% CAN BE FOUND BELOW, ATTACHED TO THIS FILE.
%
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Len Bos, Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  
%%          Len Bos          <leonardpeter.bos@univr.it>
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>       
%% Date: November 17, 2009
%--------------------------------------------------------------------------

% If "x" and "y" are row vectors, we transpose them.
if size(x,1) < size(x,2) x=x'; end
if size(y,1) < size(y,2) y=y'; end

% Computing all the recurrence terms of Chebyshev polynomials. 
ab=r_jacobi(M,-1/2,-1/2);

alpha_coeffs=ab(:,1);
beta_coeffs=ab(:,2);

a=1-y;

% Scaling coefficients to the interval [0,1].
alpha_coeffs=(alpha_coeffs+1)/2;
beta_coeffs=beta_coeffs/4;

% Q is a matrix containing the evaluation of
%               P_m(-1/2,-1/2)(x/(1-y))
% For a fixed "m", P_m(-1/2,-1/2)(x/(1-y)) is a column vector.
Q=scaled_orthogonal_polynomials(M,x,a,alpha_coeffs,beta_coeffs);

% Computing the matrix V containing all the evaluations of 
% S_{m,n}(x,y)=[P_m(-1/2,-1/2)(x/(1-y))] * [(1-y)^m] * [P_n(2m,-1/2)(y)]
% where 0 <= m+n <= M. For a fixed couple (m,n), S_{m,n}(x,y) is stored 
% as column vector.
V=[];
for m=0:M
    Vadd_term_1=Q(:,m+1);%V(:,1) is for the index relative to the degree 0.
    
    if (M-m) > 0
        ab_m=r_jacobi(M-m,2*m,-1/2);
        
        % Scaling coefficients to the interval [0,1].
        alpha_coeffs_m=(ab_m(:,1)+1)/2;
        beta_coeffs_m=ab_m(:,2)/4;    
        
        Vadd_term_2=eval_orthogonal_polynomials(y,alpha_coeffs_m,...
            beta_coeffs_m);
    else 
        Vadd_term_2=ones(size(y));
    end
    
    % Using repmat since Vadd_term_1 is a column vector, while Vadd_term_2
    % is a matrix.
    Vadd_term_1_mat=repmat(Vadd_term_1,1,size(Vadd_term_2,2));
    V=[V Vadd_term_1_mat.*Vadd_term_2];
end



function V=eval_orthogonal_polynomials(x,a,b)

%--------------------------------------------------------------------------
% OBJECT.
%---------
% Evaluation of monic orthogonal polynomial in terms of the recurrence
% coefficients.
%   P_{n+1}(x)=(x-a(n))*P_n(x)-b(n)*P_{n-1}(x)   P{-1}(x)=0, P_0(x)=1.
%--------------------------------------------------------------------------
% INPUT.
%--------
% 
% x: column vector, whose components are the points in which the scaled 
%    polynomials must be evaluated.
%
% a, b: recurrence coefficents, they determine the orthogonal monic 
%    polynomial:
%                            
%   P_{n+1}(x) = (x-alpha_coeffs(n))*P_n(x) - beta_coeffs(n)*P_{n-1}(x)
%   P_{-1}(x)=0, 
%   P_0(x)=0;
% 
%--------------------------------------------------------------------------
% OUTPUT:
%---------
%
% V: matrix containing the evaluations [P_0(x) P_1(x) ... P_...(x)].
%
%--------------------------------------------------------------------------

V(:,1)=zeros(size(x));
V(:,2)=ones(size(x));

for index=1:length(a)
    V(:,index+2)=(x-a(index)).*V(:,index+1)-(b(index))*V(:,index);
end

V=V(:,2:size(V,2));


function V=scaled_orthogonal_polynomials(n,x,a,alpha_coeffs,beta_coeffs)

%--------------------------------------------------------------------------
% INPUT.
%--------
% n: is the degree for which we evaluate the first n polynomials
%    Q_n(x) := (a^n)*P_n(x-a) by 3 terms recurrence.
% 
% x: column vector, whose components are the points in which the scaled 
%    polynomials must be evaluated.
%
% alpha_coeffs, beta_coeffs: recurrence coefficents, they determine 
%                            the orthogonal monic polynomial:
%                            P_{n+1}(x) = (x-alpha_coeffs(n))*P_n(x)
%                                        -beta_coeffs(n)*P_{n-1}(x)
%                            P_{-1}(x)=0, P_0(x)=0;
% 
%--------------------------------------------------------------------------
% OUTPUT:
%---------
%
% V: matrix containing the evaluations [P_0(x) P_1(x) ... P_n(x)].
%
%--------------------------------------------------------------------------

% If "x" is a row vector, we transpose it.
if size(x,1) < size(x,2) x=x'; end

L=length(x);

V(:,1)=zeros(size(x));
V(:,2)=ones(size(x));

for index=1:n
    a1n=a*alpha_coeffs(index);
    b1n=(a.^2)*beta_coeffs(index);

    V(:,index+2)=(x-a1n).*V(:,index+1)-b1n.*V(:,index);
end

V=V(:,2:size(V,2));












