function V=koornwinder(M,x,y)

%--------------------------------------------------------------------------
% OBJECT.
%---------
%
% Function to compute the Vandermonde of the Koornwinder Class II
% orthogonal polynomials for the bivariate unit disk.
%
%--------------------------------------------------------------------------
% INPUTS.
%---------
%
% M:   DEGREE OF THE VANDERMONDE MATRIX.
%
% x,y: POINTS OF THE UNIT DISK IN WHICH WE EVALUATE THE VANDERMONDE MATRIX.
%      "x", "y" ARE COLUMN VECTORS.
%
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
%
% V : IT IS THE VANDERMONDE MATRIX V((x,y),M) WRT THE KOORNWINDER CLASS II
% BASIS.
%
%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES.
%----------------------
%
% 1. koornwinder_vect
% 2. ScaledJacobi (called by "koornwinder_vect")
%
% THE ROUTINES ARE BELOW THE CODE OF "koornwinder".
%--------------------------------------------------------------------------
% REFERENCES.
%-------------
%
% [1] L.BOS, A. SOMMARIVA, M. VIANELLO. "Least-squares polynomial 
%         approximation on weakly admissible meshes".
%
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  
%%          Len Bos          <leonardpeter.bos@univr.it>
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 17, 2009
%--------------------------------------------------------------------------

if (size(x,1) == 1)
    x=x'; y=y';
end

V=[];

for ii=0:M
    for jj=0:ii
        Vadd=koornwinder_vect(ii,jj,x,y);
        V=[V Vadd];
    end
end

function y=koornwinder_vect(n,k,x,y)

%--------------------------------------------------------------------------
% OBJECT.
%---------
% Function to compute the Koornwinder Class II orthonormal
% polynomials for the bivariate unit disk.
%
% n is the degree and 0 \le k \le n (not checked by the code).
%--------------------------------------------------------------------------

y=ScaledJacobi(n-k,k,k,x,1).*ScaledJacobi(k,-.5,-.5,y,sqrt(1-x.^2));
v=2^k/sqrt(2*n+1);
v=v*factorial(n)/sqrt(factorial(n-k))/sqrt(factorial(n+k));
v=v*gamma(k+.5)/factorial(k);  % "v" is the normalization constant
y=y/v;


function y=ScaledJacobi(n,alpha,beta,x,a)

%--------------------------------------------------------------------------
% OBJECT.
%---------
% Function to compute (a^n)*P_n(alpha,beta,x/a)
% where P_n(alpha,beta,t) denotes the Jacobi polynomial
% of degree n. Output was checked with Maple.
%
% If a=1 then the result is just the value of the Jacobi polynomial at "x".
%--------------------------------------------------------------------------

P0=ones(size(x));
P1=(1+(alpha+beta)/2).*x+a*(alpha-beta)/2;

if n==0, y=P0; return, end
if n==1, y=P1; return, end

for k=2:n
    
    AA=(2*k-1+alpha+beta)/2/k/(k+alpha+beta);
    A=AA./(2*k-2+alpha+beta)*(alpha^2-beta^2).*a;
    B=AA/(2*k-2+alpha+beta)*(2*k+alpha+beta)*(2*k-2+alpha+beta);
    C=-(k+alpha-1)*(k+beta-1)*...
        (2*k+alpha+beta)./k./(k+alpha+beta)./(2*k-2+alpha+beta)*a.^2;
    P=(A+B*x).*P1+C.*P0;
    P0=P1;
    P1=P;
    
end
y=P;


