function V=logan_shepp(pts,polynomial_degree)

%--------------------------------------------------------------------------
% OBJECT.
%---------
%
% EVALUATION OF LOGAN-SHEPP VANDERMONDE MATRIX "V" OF DEGREE  
% polynomial_degree ON A SET OF POINTS pts OF THE UNIT DISK.
%
%--------------------------------------------------------------------------
% INPUTS.
%---------
%
% pts : SET OF POINTS OF UNIT DISK ON WHICH THE POLYNOMIALS MUST BE 
%      EVALUATED. IT IS A 2 COLUMNS MATRIX.
%
% polynomial_degree: MAXIMUM DEGREE VANDERMONDE MATRIX.
%
%---------
% OUTPUT.
%---------
% 
% V: VANDERMONDE MATRIX, WRT THE LOGAN SHEPP BASIS, EVALUATED IN THE POINTS 
%    THAT ARE THE ROWS OF THE MATRIX pts. NOTE THAT 
% 
%                  V(:)=[\phi_1(:) \phi_2(:) ... \phi_N(:)]. 
%
%   WHERE [\phi_k]_k IS THE LOGAN-SHEPP BASIS.
%
%--------------------------------------------------------------------------
% FUNCTIONS USED BY THIS ROUTINE.
%---------------------------------
% 
% 1. logan_shepp_jn
% 2. chebyshev_second_type (USED BY "logan_shepp_jn").
%
% THE FUNCTIONS ARE ATTACHED BELOW.
%
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 17, 2009
%--------------------------------------------------------------------------

V=[];
for n=0:polynomial_degree
    j=(0:n)';
    V_loc=logan_shepp_jn(pts,j,n);
    V=[V V_loc];
end




function fx=logan_shepp_jn(pts,j,n)

%--------------------------------------------------------------------------
% INPUTS.
%---------
%
% pts: IT IS A MATRIX N x 2, SUCH THAT pts_k=(x_k,y_k).
%
% j: IT IS A COLUMN VECTOR M x 1 WHOSE COMPONENTS ARE INTEGERS, LESS OR 
%    EQUAL TO n (SEE THIRD INPUT OF logan_shepp).
%
% n: LOGAN-SHEPP DEGREE.
%
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
%
% fx: EVALUATION OF j-th LOGAN-SHEPP POLYNOMIAL OF ORDER n AT pts. 
%     IT IS A MATRIX, WITH N ROWS AND M COLUMNS.
%
%--------------------------------------------------------------------------
% FUNCTIONS USED BY THIS ROUTINE.
%---------------------------------
% 
% 1. chebyshev_second_type (USED BY "logan_shepp_jn").
%
% THE FUNCTION IS ATTACHED BELOW.
%
%--------------------------------------------------------------------------

normalized_ls=0; % 0: normalized, 1: not normalized.

x=pts(:,1); y=pts(:,2);

M=size(j,1);
N=size(pts,1);

x=repmat(x,M,1); y=repmat(y,M,1);
j=repmat(j,1,N); j=j'; j=j(:);

args= x.*cos( j*pi/(n+1) ) + y.*sin( j*pi/(n+1) );

if normalized_ls == 0
    fx=(1/sqrt(pi))*chebyshev_second_type(args,n);
else
    fx=chebyshev_second_type(args,n);
end

fx=reshape(fx,N,M);




function fx=chebyshev_second_type(x,n)

%--------------------------------------------------------------------------
% INPUTS.
%---------
%
% x: POINTS.
%
% n: SECOND TYPE CHEBYSHEV POLYNOMIAL DEGREE.
%
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
%
% fx: EVALUATION OF SECOND TYPE CHEBYSHEV POLYNOMIAL OF ORDER n AT POINTS
% x.
%
%--------------------------------------------------------------------------

method=2;

switch method 
case 1
    theta=acos(x);
    s=sin(theta);
    s=s+(1-abs(sign(s)))*eps;
    fx=sin((n+1)*theta)./s;
case 2
    
    % WE COMPUTE RECURSIVELY U_n=sin((n+1)*theta)./s.
    un_2=ones(size(x));
    un_1=2*x;
    
    switch n
    case 0
        fx=un_2;
    case 1
        fx=un_1;
    otherwise
        
        for index=2:n
            un=2*x.*un_1-un_2;
            un_2=un_1;
            un_1=un;
        end
        fx=un;
    end
    
end


 