
function [nodes_fek_x,nodes_fek_y,nodes_fek_cub_x,nodes_fek_cub_y,...
    w_fek_cub,leb_const,ls_norm]=main_fekete(M,basis_type,domain_type,...
    polygon_sides,spline_order_vett,SPLtypestring,rotation,...
    P,Q,cubature_type,cumulative,wam_select,niter,function_type,...
    output_parms,funfcn1,funfcn2,a_trap,b_trap,extraction_method);


%--------------------------------------------------------------------------
% THE MAIN PURPOSE OF THIS ROUTINE IS TO (POSSIBLY) COMPUTE
% 1. APPROXIMATION OF FEKETE POINTS FOR INTERPOLATION
%    (nodes_fek_x,nodes_fek_y) AND CUBATURE (cub_x,nodes_fek_cub_y) (AND
%     RELATIVE WEIGHTS w_fek_cub), FOR A DOMAIN DETERMINED BY domain_type
%    (SEE define_domain.m).
% 2. PLOT WAM POINTS AND APPROXIMATE FEKETE POINTS.
% 3. COMPUTE LEBESGUE CONSTANTS AND LEAST SQUARES NORMS.
%--------------------------------------------------------------------------
% INPUTS.
%----------
%
% M : DEGREE OF THE VANDERMONDE BASIS.
%
% basis_type: IT DETERMINES THE BASIS FOR THE DOMAIN.
%
%    [basis_type = 0] MONOMIAL (NOT SCALED).
%    [basis_type = 1] LEGENDRE (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                     CONTAINING THE DOMAIN.
%    [basis_type = 2] CHEBYSHEV (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                     CONTAINING THE DOMAIN.
%    [basis_type = 3] MONOMIAL (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                     CONTAINING THE DOMAIN.
%    [basis_type = 4] ORTHONORMAL DUBINER BASIS FOR UNIT TRIANGLE
%                     [0 0; 1 0; 0 1].
%    [basis_type = 5] LOGAN SHEPP BASIS FOR UNIT DISC B(C=(0,0), r=1).
%    [basis_type = 6] SCALED MONOMIAL BASIS FOR THE INTERVAL.
%    [basis_type = 7] SCALED LEGENDRE BASIS FOR THE INTERVAL.
%    [basis_type = 8] SCALED CHEBYSHEV BASIS FOR THE INTERVAL.
%    [basis_type = 9] KOORNWINDER BASIS FOR UNIT DISC B(C=(0,0), r=1).
%    [basis_type = 10] DUBINER ORTHOGONAL BASIS FOR UNIT SIMPLEX W.R.T.
%                      THE EQUILIBRIUM MEASURE.
%    [basis_type =11] FAST, SCALED CHEBYSHEV VANDERMONDE.

% domain_type: IT DETERMINES THE DOMAIN AS DESCRIBED IN THE M-FILE
%              "define_domain.m". SOME EXAMPLES ARE:
%
%    [polygon_type=0]: UNIT DISK (parametrized as disc).
%    [polygon_type=1]: UNIT SQUARE [0,1]^2.
%    [polygon_type=2]: A 6 SIDES CONVEX POLYGON USED IN PAPER [1].
%    [polygon_type=3]: A 9 SIDES NONCONVEX POLYGON USED IN THE PAPER [1].
%    [polygon_type=4]: A 9 SIDES POLYGON.
%    [polygon_type=5]: A 10 SIDES POLYGON.
%    [polygon_type=6]: UNIT SQUARE [-1,1]^2.
%    [polygon_type=7]: UNIT TRIANGLE [0 0; 1 0; 0 1; 0 0].
%    [polygon_type=8]: ECLYPSE.
%    [polygon_type=9]: POINTS ON MONITOR.
%    [polygon_type=10]: CIRCLE (APPROXIMATED BY SPLINE POLYGON).
%    [polygon_type=11]: BOOMERANG
%    [polygon_type=12]: QUADRANGLE
%    [polygon_type=13]: NON CONVEX POLYGON, 10 VERTICES.
%    [polygon_type=14]: NON CONVEX POLYGON, HAND');
%    [polygon_type=15]: UNIT TRIANGLE [0 0; 1 0; 1 1; 0 0].
%    [polygon_type=16]: RANDOM TRIANGLE.
%    [polygon_type=17]: AN HEXAGON.
%    [polygon_type=15]: INTERVAL [-1,1].
%
%    TO DEFINE YOUR OWN POLYGON, PLEASE OPEN THE m-FILE "define_domain.m"
%    AND ADD IT TO THE LIST. OBSERVE THAT THE FIRST VERTEX IS NOT REPEATED.
%    FOR EXAMPLE, THE UNIT SQUARE IS DEFINED AS "[0 0; 1 0; 1 1; 0 1]"
%    AND NOT "[0 0; 1 0; 1 1; 0 1; 0 0]".
%
% polygon_sides : THE DEFINITION OF THE DOMAIN COMES FROM DETERMINATING
%    SOME SAMPLINGS OF THE DOMAIN.
%    MORE PRECISELY  IF THE POLYGON HAS "L" SIDES, "polygon_sides"
%    IS A VARIABLE CONTAINING ITS VERTICES, ORDERED COUNTERCLOCKWISE.
%    AS LAST ROW MUST HAVE THE COMPONENTS OF THE FIRST VERTEX.
%
%    IN OTHER WORDS, THE FIRST ROW AND LAST ROW ARE EQUAL.
%
%    "polygon_sides" IS A "L+1 x 2" MATRIX.
%                  IN THIS DEMO, THE POLYGON IS DEFINED BY THE ROUTINE
%                  "define_domain.m".
%
% spline_order_vett:SPLINE ORDER AND BLOCK (VECTOR).
%
%    [spline_order_vett(i,1)=2] : PIECEWISE LINEAR SPLINES.
%    [spline_order_vett(i,1)=4] : CUBIC SPLINES
%                                (DEPENDING ON "spline_type" IN INPUT).
%    [spline_order_vett(i,1)=k] : IN THE CASE k IS NOT 2 OR 4 IT
%                                 CHOOSES THE k-TH ORDER SPLINES
%                                 (FOR ADDITIONAL HELP DIGIT "help spapi"
%                                 IN MATLAB SHELL).
%
%    [spline_order_vett(:,2)]  : VECTOR OF FINAL COMPONENTS OF A BLOCK.
%
%    EXAMPLE:
%
%    "spline_order_vett=[2 31; 4 47; 8 67]" MEANS THAT FROM THE 1st
%    VERTEX TO THE 31th VERTEX WE HAVE AN ORDER 2 SPLINE (piecewise
%    linear), FROM THE 32th VERTEX TO THE 47th WE USE A 4th ORDER
%    SPLINE (i.e. A CUBIC AND PERIODIC SPLINE BY DEFAULT), FROM THE
%    48th TO THE 67th (AND FINAL!) WE USE AN 8th ORDER SPLINE.
%
% SPLtypestring: IF THE MATRIX (M X 2) "spline_order_vett" HAS SOME ELEMENT
%   IN THE FIRST COLUMN EQUAL TO ONE, SPLtypestring IT DECIDES THE TYPE OF
%   END CONDITIONS OF THE CUBIC SPLINE IN THE PERTINENT BLOCK.
%   IT IS A STRING. IT CAN BE:
%
%   complete'   : match endslopes (as given in VALCONDS,
%                 with default as under *default*).
%   'not-a-knot' : make spline C^3 across first and last
%                  interior break (ignoring VALCONDS if given).
%   'periodic'   : match first and second derivatives at
%                 first data point with those at last data
%                 point (ignoring VALCONDS if given).
%   'second'     : match end second derivatives (as given
%                  in VALCONDS,with default [0 0], i.e.,
%                 as in variational).
%   'variational': set end second derivatives equal to zero
%                  (ignoring VALCONDS if given).

% rotation: SUPPOSE x_min, x_max, y_min, y_max ARE THE MINIMUM AND MAXIMUM
%    VALUES IN x AND y REACHED BY THE POLYGON, I.E.
%    "R=[x_min,x_max] x [y_min,y_max]" IS THE SMALLEST RECTANGLE WITH
%    SIDES PARALLEL TO THE AXIS x AND y, CONTAINING THE POLYGON.
%
%   [rotation=0]: GENERAL CASE, BUT OBSERVE THAT THE FUNCTION MUST BE DEFINED
%                 IN THE RECTANGLE "R" DESCRIBED ABOVE.
%   [rotation=1]: GOOD FOR CONVEX POLYGONS. FOR CONVEX POLYGONS IT SUFFICES
%                 THAT THE FUNCTION IS DEFINED IN THE POLYGON.
%   [rotation=2]: THE USER CHOOSES A SPECIAL REFERENCE SEGMENT "PQ".
%                 CHECK [1] FOR FURTHER DETAILS. SEE THE VARIABLES "P", "Q"
%                 BELOW.
%
% P,Q: IF "rotation=2" THEN THE ALGORITHM CHOOSES A PREFERRED SEGMENT
%            "PQ" HAVING "P", "Q" AS EXTREMA, AS SET BY THE USER.
%            "P" AND "Q" ARE "1 x 2" ROW VECTORS. (SEE [3] FOR DETAILS).
%
% cubature_type: IT CHOOSES THE QUADRATURE RULE USED BY SPLINEGAUSS.
%
%   [cubature_type=0]: PADUA POINTS OVER SQUARE PANELS.
%   [cubature_type=1]: TENSORIAL FEJER 1.
%   [cubature_type=2]: TENSORIAL FEJER 2.
%   [cubature_type=3]: TENSORIAL CLENSHAW CURTIS (VIA WEIGHTS).
%   [cubature_type=4]: TENSORIAL GAUSS-LEGENDRE.
%   [cubature_type=5]: TENSORIAL GAUSS-LEGENDRE-LOBATTO.
%
% cumulative: IT CHOOSES THE PARAMETRIZATION OF THE BOUNDARY WHEN IS
%   DEFINED PARAMETRICALLY VIA SPLINES (SEE [3] FOR DETAILS).
%
%   [cumulative=0]: 1:N.
%   [cumulative=1]: CUMULATIVE.
%
% wam_select: IT SELECTS THE WAM.
%
%   [wam_select] -7: Interval. 1/N^2 equispaced points.
%   [wam_select] -6: Simplex. New mesh v.2.0. (Oct. 2009).
%   [wam_select] -5: Admissible mesh from tens. grid for disc (and find).
%   [wam_select] -4: Points for a trapezoidal region (tensorial pts based).
%   [wam_select] -3: Admissible mesh from tens. grid (and inpolygon).
%   [wam_select] -2: Points for simplex (2008).
%   [wam_select] -1: Points for a trapezoidal region (Padua pts based).
%   [wam_select]  0: Points for a disk (2008-2009).
%   [wam_select]  1: Padua type panels over curvilinear polygons (via
%                    splinegauss_2009).
%   [wam_select]  2: Gaussian or Clenshaw Curtis or Fejer type
%                    over curvilinear polygons (via splinegauss_2009).
%   [wam_select]  3: Minimal triangulation WAM (pts of a reference rectangle
%                    are mapped in any triangle of the triangulation).
%   [wam_select]  4: Minimal triangulation WAM (pts of a reference triangle
%                    WAM are mapped in any triangle of the triangulation).
%   [wam_select]  5: Barycentric triangulation WAM (pts of a reference
%                    triangle WAM are mapped in any triangle of the
%                    triangulation). Good for star shaped or covex domains.
%
% niter: NUMBER OF TIMES THAT THE DISCRETE ORTHOGONALIZATION PROCESS TAKES
%        PLACE. IF niter=0 THEN NO DISCRETE ORTHOGONALIZATION.
%
% function_type: TEST FUNCTION TO BE APPROXIMATED.
%
%                1: .75*exp(-((9*x-2).^2 + (9*y-2).^2)/4) + ...
%                    .75*exp(-((9*x+1).^2)/49 - (9*y+1)/10) + ...
%                    .5*exp(-((9*x-7).^2 + (9*y-3).^2)/4) - ...
%                    .2*exp(-(9*x-4).^2 - (9*y-7).^2);
%
%                2: ( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
%                3: (x+y).^20;
%                4: exp((x-0.5).^2+(y-0.5).^2);
%                5: exp(-100*((x-0.5).^2+(y-0.5).^2));
%                6: cos(30*(x+y));
%                7: ones(size(x));
%                8: exp(x+y);
%                9: 1./[1+16*(x.^2+y.^2)];
%                10: (x.^2+y.^2).^(3/2);
%                11: exp(-x.^2-y.^2);
%                12: exp(-[ (x.^2+y.^2) ]^(-1))
%                13: cos(x.^2+y.^2);
%                14: cos((x+y));
%                15: ((x-0.4)^2 +(y-0.4)^2 )^(1/2);
%
% output_parms: THIS MATLAB CODE PERFORMS SEVERAL PURPOSES. IN THIS PART
%   YOU CAN SELECT OUTPUTS ONLY FOR THOSE IN WHICH YOU ARE INTERESTED.
%
%   0: NO 1: YES.
%
%   [output_parms(1) ]: LEBESGUE CONSTANT.
%   [output_parms(2) ]: VANDERMONDE CONDITIONING.
%   [output_parms(3) ]: IT PLOTS THE DOMAIN AND THE W.A.M. NODES.
%   [output_parms(4) ]: VANDERMONDE DETERMINANTS.
%   [output_parms(5) ]: INTERPOLATION STATS.
%   [output_parms(6) ]: CUBATURE STATS.
%
%   EXAMPLE.
%
%   output_parms = [0 0 0 0 1 1];
%
%   GIVES THE VALUES OF VANDERMONDE CONDITIONING, IT PLOTS THE DOMAIN AND
%   THE W.A.M. NODES, IT GIVES SOME VANDERMONDE DETERMINANTS, INTERPOLATION &
%   CUBATURE STATISTICS.
%
% extraction_method: 1: QR 2:LU.
%
% funfcn1,funfcn2,a_trap,b_trap: IF [polygon_type=-1] THE TRAPEZOIDAL REGION IS
%   DESCRIBED BY A SMALLER VALUED FUNCTION "g1" AND A BIGGER ONE "g2",
%   AND THE "x" INTERVAL [a_trap,b_trap].
%
%   EXAMPLES:
%   1. a_trap=-1; b_trap=1; g1=inline('x'); g2=inline('x+3');
%   2. a_trap=-1; b_trap=1; g1=inline('-ones(size(x))');
%      g2=inline('x/4+3/4');
%   3. a_trap=-1; b_trap=1; g1=inline('x'); g2=inline('x+3');
%
% a, b, c, d: THEY DETERMINE THE SMALLER SQUARE INCLUDING THE DOMAIN (AT
%    LEAST NUMERICALLY) WITH SIDES PARALLEL TO THE CARTESIAN AXIS. IT IS
%    USED TO SCALE SOME BASIS.
%
%----------
% OUTPUT:
%----------
%
% nodes_fek_x,nodes_fek_y: APPROXIMATE FEKETE POINTS (FOR INTERPOLATION, IF
%    REQUIRED).
%
% nodes_fek_cub_x,nodes_fek_cub_y: APPROXIMATE FEKETE POINTS (FOR
%    CUBATURE, IF REQUIRED).
%
% leb_const,ls_norm: APPROXIMATE LEBESGUE CONSTANTS, LEAST SQUARES NORM (IF
%    REQUIRED).
%
%--------------------------------------------------------------------------
% REFERENCE PAPERS.
%-------------------
%
% [1] L. BOS and N. LEVENBERG
%     "On the Approximate Calculation of Fekete Points: the Univariate Case"
%     ETNA.
%
% [2] L. BOS, N. LEVENBERG, J.P. CALVI, A. SOMMARIVA and M. VIANELLO
%     "Geometric Weakly Admissible Meshes, Discrete Least Squares 
%      Approximations and Approximate Fekete Points". Math. Comp. (to
%      appear).
%
% [3] L.BOS, A. SOMMARIVA, M. VIANELLO. "Least-squares polynomial 
%         approximation on weakly admissible meshes". Preprint.
%
% [4] J.P. CALVI and N. LEVENBERG
%     "Uniform approximation by discrete least squares polynomials".
%     J. Approx. Theory 152 (2008), 82-100.
%
% [5] A. SOMMARIVA and M. VIANELLO
%     "Gauss-Green cubature over spline curvilinear polygons". 
%     J. Comp. Appl. Math.
%
% [6] A. SOMMARIVA and M. VIANELLO
%     "Computing approximate Fekete points by QR factorizations
%     of Vandermonde matrices". Comp. Math. Appl. 
%
%--------------------------------------------------------------------------
% FUNCTIONS USED IN THIS ROUTINE.
%---------------------------------
%
% 1. wam
% 2. fekete_block
% 3. vandermonde_2D
% 4. fct2D
% 5. computing_moments
% 6. cubature_2D
%
% ALL THESE FUNCTIONS ARE EXTERNAL TO THIS ROUTINE.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 17, 2009
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
%               ...... HERE THE MAIN PROGRAM BEGINS ......
%-------------------------------------------------------------------------
% functions needed only for trapezoidal regions.
g1 = fcnchk(funfcn1); g2 = fcnchk(funfcn2);

V_fek=[]; leb_const=[]; ls_norm=[]; % INITIALIZATIONS.

%--------------------------------------------------------------------------
% GENERATING POINTS OF THE WEAKLY ADMISSIBLE MESH. THE SET IS GIVEN AS
% [nodes_x,nodes_y]. WEIGHTS "weights" OF A CUBATURE RULE ARE ALSO PROVIDED.
%--------------------------------------------------------------------------

% COMPUTING WAM.
[nodes_x,nodes_y,weights,N,a,b,c,d,tri,boundary_pts]=wam(M,polygon_sides,...
    spline_order_vett,SPLtypestring,rotation,P,Q,cubature_type,...
    cumulative,wam_select,g1,g2,a_trap,b_trap);

% WARNING: LARGE WAM.
if length(nodes_x) > 40000
    
    fprintf('\n \n \t >> [WARNING]: LARGE WAM (%5.0f POINTS).',...
        length(nodes_x));
    fprintf('\n \t               THE ALGORITHM MAY BE TIME CONSUMING.');
    
    answer=input('\n \n >> PRESS "1" TO CONTINUE, "0" TO STOP: ');
    if answer == 1
    else
        return
    end
      
end

pts_wam=[nodes_x nodes_y];

% VANDERMONDE AT WAM POINTS.
V_wam=vandermonde_2D(basis_type,pts_wam,M,a,b,c,d);

% COMPUTING APPROXIMATE FEKETE POINTS.

[fek,ind,w_intp,C,Pmat]=computing_fekete_points(pts_wam,V_wam,...
    niter,[],extraction_method);

%--------------------------------------------------------------------------
% INTERPOLATION.
%--------------------------------------------------------------------------

if output_parms(5) > 0

    % VANDERMONDE FOR INTP AT APPR. FEKETE PTS.
    V_fek=vandermonde_2D(basis_type,fek,M,a,b,c,d);

    f_fek=fct2D(fek(:,1),fek(:,2),function_type);

    % SOLVING LINEAR SYSTEM TO COMPUTE INTP. COEFFICIENTS.
    coeffs_fek=V_fek\f_fek;

end

%--------------------------------------------------------------------------
% LEAST SQUARES.
%--------------------------------------------------------------------------

% COMPUTING LEAST SQUARES AT WAM POINTS.

if output_parms(5) > 0

    % VANDERMONDE FOR INTP AT WAM POINTS.
    V_ls=vandermonde_2D(basis_type,[nodes_x nodes_y],M,a,b,c,d);

    % FUNCTION EVALUATION.
    f_nodes=fct2D(nodes_x,nodes_y,function_type);

    % LEAST SQUARE SOLUTION BY ITERATION (IF REQUIRED).
    coeffs_ls=Pmat*((V_ls*Pmat)\f_nodes);
    
    if isempty(boundary_pts) == 0
                
        n_pts=size(V_ls,1);
        other_pts=setdiff(1:n_pts,boundary_pts);
        
        intp_V=V_ls(boundary_pts,:); 
        intp_V=intp_V*Pmat;
        ls_V=V_ls(other_pts,:);
        ls_V=ls_V*Pmat;
        
        f_nodes_intp=f_nodes(boundary_pts);
        f_nodes_ls=f_nodes(other_pts);
        
        % coeffs_lsqlin=Pmat*((V_ls*Pmat)\f_nodes);
        coeffs_lsqlin = Pmat*lsqlin(ls_V,f_nodes_ls,[],[],intp_V,f_nodes_intp);
        
        V_ls_bd=vandermonde_2D(basis_type,[nodes_x(boundary_pts) nodes_y(boundary_pts)],M,a,b,c,d);
        f_lsqlin_testing=V_ls_bd*coeffs_lsqlin;
        
        norm(f_lsqlin_testing-f_nodes_intp,inf)
        pause
        
    else
        
        coeffs_lsqlin=[];
        
    end

end

%--------------------------------------------------------------------------
% CUBATURE.
%--------------------------------------------------------------------------

% COMPUTING ALGEBRAIC CUBATURE BY EXTRACTION FROM WAM POINTS.

if output_parms(6) > 0

    % COMPUTING MOMENTS.
    moms=computing_moments(M,basis_type,domain_type,polygon_sides,...
        spline_order_vett,SPLtypestring,rotation,P,Q,cubature_type,...
        cumulative,a_trap,b_trap,g1,g2,a,b,c,d);

    % COMPUTING FEKETE POINTS "fek_cub" AND WEIGHTS "w_fek_cub".
    [fek_cub,ind_cub,w_fek_cub,C_cub,Pmat_cub]=computing_fekete_points(...
        pts_wam,V_wam,niter,moms,extraction_method);

    % FUNCTION EVALUATION AT CUBATURE PTS.
    f_cub_nodes=fct2D(fek_cub(:,1),fek_cub(:,2),function_type);

    % EVALUATING CUBATURE FORMULA.
    cub_fek=w_fek_cub'*f_cub_nodes;

    % VANDERMONDE FOR RESIDUAL EVALUATION VANDERMONDE AFEK vs MOMS.
    if output_parms(5) <= 0
        V_fek=vandermonde_2D(basis_type,fek,M,a,b,c,d);
    end

    residual_fek=norm(V_fek'*w_fek_cub-moms); % Residual.
else
    w_fek_cub=[];
end

% -------------------------------------------------------------------------
% TESTING ERRORS.
%--------------------------------------------------------------------------

%-----------------------
% APPROXIMATION ERRORS.
%-----------------------

if output_parms(5) > 0

    % TEST PTS.
    [test_nodes_x,test_nodes_y,test_weights,test_N,a_test,b_test,...
        c_test,d_test,tri]=wam(M+1,polygon_sides,spline_order_vett,...
        SPLtypestring,rotation,P,Q,cubature_type,...
        cumulative,wam_select,g1,g2,a_trap,b_trap);

    % VANDERMONDE MATRIX AT TEST POINTS.
    V_test=vandermonde_2D(basis_type,[test_nodes_x test_nodes_y],M,a,b,c,d);

    % FUNCTION EVALUATION.
    f_test=fct2D(test_nodes_x,test_nodes_y,function_type);

    % EVALUATION OF THE INTERPOLANT AT TEST POINTS.
    f_fek_test=V_test*coeffs_fek;

    % EVALUATION LEAST SQUARES APPROXIMANT AT TEST POINTS.
    f_ls_test=V_test*coeffs_ls;
    
    if isempty(coeffs_lsqlin) == 0
        f_lsqlin_test=V_test*coeffs_lsqlin;
        abserr_inf_lsqlin_test=norm(f_lsqlin_test-f_test,inf);
        aver_abserr_2_lsqlin_test=norm(f_lsqlin_test-f_test,2)/sqrt(length(f_fek_test));
    else
        f_lsqlin_test=[];
        abserr_inf_lsqlin_test=[];
        aver_abserr_2_lsqlin_test=[];
    end

    % EVALUATION ABSOLUTE ERRORS IN INF. NORM.
    abserr_inf_fek_test=norm(f_fek_test-f_test,inf);
    abserr_inf_ls_test=norm(f_ls_test-f_test,inf);
    
    
    % EVALUATION ABSOLUTE ERRORS IN WEIGHTED 2 NORM.
    aver_abserr_2_fek_test=...
        norm(f_fek_test-f_test,2)/sqrt(length(f_fek_test));
    aver_abserr_2_ls_test=norm(f_ls_test-f_test,2)/sqrt(length(f_fek_test));
    

end

%-----------------------
% CUBATURE ERRORS.
%-----------------------

if output_parms(6) > 0

    % "REFERENCE" CUBATURE: NODES AND WEIGHTS.
    [nodes_x_exact,nodes_y_exact,weights]=cubature_2D(basis_type,...
        domain_type,polygon_sides,spline_order_vett,SPLtypestring,...
        rotation,P,Q,cubature_type,cumulative,a_trap,b_trap,g1,g2);

    % FUNCTION EVALUATION FOR EXACT CUBATURE.
    f_pts_exact=fct2D(nodes_x_exact, nodes_y_exact,function_type);

    % CUBATURE RESULT BY SCALAR PRODUCT.
    cub_approx=weights'*f_pts_exact;
    cub_err=abs(cub_approx-cub_fek);

end

nodes_fek_x=fek(:,1);
nodes_fek_y=fek(:,2);

if output_parms(6) > 0

    nodes_fek_cub_x=fek_cub(:,1);
    nodes_fek_cub_y=fek_cub(:,2);

else

    nodes_fek_cub_x=[];
    nodes_fek_cub_y=[];

end

%--------------------------------------------------------------------------
% STATISTICS.
%--------------------------------------------------------------------------

%-------------------
% BASIC STATISTICS.
%-------------------
fprintf('\n \n \t [DOMAIN]: %2.0f', domain_type);
fprintf(' [DEGREE]: %2.0f', M);
fprintf(' [1D WAM]: %2.0f', cubature_type);
fprintf(' [BASIS ]: %2.0f', basis_type);
fprintf('\n \t [ITERS.]: %2.0f', niter);
fprintf(' [AF.PTS]: %2.0f', size(fek,1));
fprintf(' [IN.PTS]: %2.0f', size(nodes_x,1));
fprintf(' [WAM]: %2.0f', wam_select);
if (extraction_method == 1)
    fprintf('\n \t [EXTRACTION METHOD]: QR');
else
    fprintf('\n \t [EXTRACTION METHOD]: LU');
end

%----------------------
% APPROXIMATION ERRORS.
%----------------------
if output_parms(5) > 0
    fprintf('\n \n \t >> APPROXIMATION STATS. \n');
    fprintf('\n \t [FUNCTION]: %2.0f',function_type);
    fprintf('\n \t [ABS.ERR.APPROX.FEK.LS.(INF.NORM.)]: %2.0e',...
        abserr_inf_ls_test);
    
    if isempty(abserr_inf_lsqlin_test) == 0
    fprintf('\n \t [ABS.ERR.APPROX.FEK.LSQLIN.(INF.NORM.)]: %2.0e',...
        abserr_inf_lsqlin_test);
    end
    
    fprintf('\n \t [ABS.ERR.APPROX.FEK.INTP.(INF.NORM.)]: %2.0e',...
        abserr_inf_fek_test);
end

%-----------------
% CUBATURE ERRORS.
%-------------------
if output_parms(6) > 0
    fprintf('\n \n \t >> CUBATURE STATS. \n');
    fprintf('\n \t [ABS.CUB.ERR.]: %2.1e',cub_err);
    fprintf('\n \t [APPROX.CUB. ]: %5.15e',cub_approx);
    fprintf('\n \t [SUM ABS. WEIGHTS]: %2.2e',sum(abs(w_fek_cub)));
    fprintf('\n \t [SUM WEIGHTS]: %2.12e',sum(w_fek_cub));
    fprintf('\n \t [CUB. COND.]: %2.2e',...
        abs(sum(abs(w_fek_cub))/sum(w_fek_cub)));
end

%------------------------------------------
% LEBESGUE CONSTANT AND LEAST SQUARES NORM.
%------------------------------------------

%**************************************************************************
% Here we compute the Lebesgue constant and the least square norm, by
% evaluating the Lebesgue function and the norms on wams of larger degree. 
% In case these wams are too large, we try to evaluate the Lebesgue
% constant and the least square norm on small meshes.
%**************************************************************************

leb1=[]; ls_norm=[];
if output_parms(1) > 0
    fprintf('\n \n \t >> LEBESGUE - LEAST SQUARES STATS. ');
    leb_M=[M 2*M 3*M 4*M 5*M]; % INCREASING sequence of M.

    if output_parms(5) <= 0
        V_ls=vandermonde_2D(basis_type,[nodes_x nodes_y],M,a,b,c,d);
    end 
    
    large_mesh=0;

    for ii=1:length(leb_M)
        
        if (leb_M(ii) <= 100) & (large_mesh == 0) 
            
            [test_leb_nodes_x,test_leb_nodes_y,test_leb_weights,N,...
                a,b,c,d,tri]=wam(leb_M(ii),polygon_sides,...
                spline_order_vett,SPLtypestring,rotation,P,Q,...
                cubature_type,cumulative,wam_select,g1,g2,a_trap,b_trap);

            if length(test_leb_nodes_x) < 10000

                V_leb_test=vandermonde_2D(basis_type,...
                    [test_leb_nodes_x test_leb_nodes_y],M,a,b,c,d);

                C_leb_test=V_leb_test*Pmat;

                leb1(ii)=norm(C(:,ind)\C_leb_test',1);
                wam_size(ii)=length(test_leb_nodes_x);
                ls_norm(ii)=norm((V_ls*Pmat)*(V_leb_test*Pmat)',1);

            else
                
                large_mesh=1;
                
            end

        end

    end

    if length(leb1) == 0
        
        fprintf('\n \t [WARNING]: LARGE WAMS. RESULTS MAY BE INACCURATE.');

        leb_M=[2 3 4];

        for ii=1:length(leb_M)

            if leb_M(ii) <= 100

                [test_leb_nodes_x,test_leb_nodes_y,test_leb_weights,N,...
                    a,b,c,d,tri]=wam(leb_M(ii),polygon_sides,...
                    spline_order_vett,SPLtypestring,rotation,P,Q,...
                    cubature_type,cumulative,wam_select,g1,g2,a_trap,...
                    b_trap);

                if length(test_leb_nodes_x) < 10000

                    V_leb_test=vandermonde_2D(basis_type,...
                        [test_leb_nodes_x test_leb_nodes_y],M,a,b,c,d);

                    C_leb_test=V_leb_test*Pmat;

                    leb1(ii)=norm(C(:,ind)\C_leb_test',1);
                    wam_size(ii)=length(test_leb_nodes_x);
                    ls_norm(ii)=norm((V_ls*Pmat)*(V_leb_test*Pmat)',1);

                end

            end

        end

    end


    fprintf('\n \n \t [LEB. CONST. AT APPR. FEK. PTS]');
    for ii=1:length(leb1)
        fprintf('\n \t [M]: %4.0f [PTS]: %6.0f [LEB]: %2.3e',...
            leb_M(ii),wam_size(ii),leb1(ii));
    end

    fprintf('\n \n \t [LEAST SQUARES INF NORM]');
    for ii=1:length(ls_norm)
        fprintf('\n \t [M]: %4.0f [PTS]: %6.0f [NRM]: %2.3e',...
            leb_M(ii),wam_size(ii),ls_norm(ii));
    end

    leb_const=max(leb1);
    ls_norm=max(ls_norm);
    
end

%------------------------------------------
% CONDITIONING OF THE VANDERMONDE MATRIX 
% (AFTER A POSSIBLE ORTHOGONALIZATION).
%------------------------------------------
if output_parms(2) > 0

    fprintf('\n \n \t >> VANDERMONDE CONDITIONING. \n');

    % spectral condition of the relevant Vandermonde matrices
    vander_cond=cond(C);
    extract_cond=cond((C(:,ind))');
    fprintf('\n \t [VANDERMONDE COND. FULL WAM]: %2.2e',vander_cond);
    fprintf('\n \t [VANDERMONDE COND. AFEK.PTS]: %2.2e',extract_cond);

end


%-----------------------------------------------------
% VANDERMONDE MATRICES AT FEKETE POINTS. CONDITIONING.
%-----------------------------------------------------
if output_parms(4) > 0

    fprintf('\n \n \t >> VANDERMONDE DETERMINANT. \n');

    if output_parms(3) <= 0
        a=min([polygon_sides(:,1); nodes_x]);
        b=max([polygon_sides(:,1); nodes_x]);
        c=min([polygon_sides(:,2); nodes_y]);
        d=max([polygon_sides(:,2); nodes_y]);
    end


    % spectral condition of the relevant Vandermonde matrices
    switch domain_type

        case 0

            %--------------------------------------------------------------
            % UNIT REFERENCE DISK.
            % ORTHONORMAL BASIS (LOGAN-SHEPP).
            %--------------------------------------------------------------
            fprintf('\n \t [BASIS]: KOORNWINDER CLASS II (DISK)')
            Viter=koornwinder(M,fek(:,1),fek(:,2));

        case 7

            %--------------------------------------------------------------
            % UNIT REFERENCE TRIANGLE [0 0; 1 0; 0 1].
            % ORTHONORMAL DUBINER BASIS.
            %--------------------------------------------------------------
            fprintf('\n \t [BASIS]: DUBINER (EQUILIBRIUM) (SIMPLEX)')
            Viter=dubiner_simplex_equilibrium(M,fek(:,1),fek(:,2));

        case 18
            switch basis_type

                case 8

                    %------------------------------------------------------
                    % INTERVAL.
                    % SCALED CHEBYSHEV BASIS.
                    %------------------------------------------------------
                    fprintf('\n \t [BASIS]: SCALED CHEBYSHEV (INTERVAL)');
                    Viter=vandermonde_1D(2,fek(:,1),M,a,b);

                case 7

                    %------------------------------------------------------
                    % INTERVAL.
                    % SCALED LEGENDRE BASIS.
                    %------------------------------------------------------
                    fprintf('\n \t [BASIS]: SCALED CHEBYSHEV (INTERVAL)');
                    Viter=vandermonde_1D(1,fek(:,1),M,a,b);

                case 6

                    %------------------------------------------------------
                    % INTERVAL.
                    % SCALED MONOMIAL BASIS.
                    %------------------------------------------------------
                    fprintf('\n \t [BASIS]: MONOMIAL (INTERVAL)')
                    Viter=vandermonde_1D(3,fek(:,1),M,a,b);
            end

        otherwise

            %--------------------------------------------------------------
            % CASE 2: basis_type=1, 2, 3:
            % APPROX. COMPUTING SMALLEST SQUARE [a,b]x[c,d] CONTAINING
            % THE DOMAIN.
            % IT IS APPROXIMATE SINCE THE BOUNDARY CAN BE A CURVILINEAR ONE
            % (OR 1D POINTS MAY BE INSIDE THE INTERVAL).
            % IN THIS CASE A MORE CLEVER APPROACH SHOULD BE USED.
            %--------------------------------------------------------------
            fprintf('\n \t [BASIS]: SCALED CHEBYSHEV (GENERAL DOMAIN)')
            a=min(nodes_x); b=max(nodes_x); c=min(nodes_y); d=max(nodes_y);
            Viter=vandermonde_polygons(2,fek,M,a,b,c,d);

    end

    if size(Viter,1) == size(Viter,2)
        fprintf('\n \t [VANDERMONDE DET]: FEKETE POINTS: %2.2e',det(Viter));
    else
        fprintf('\n \t [VANDERMONDE DET]: FEKETE POINTS: %2.2e', realmax);
    end

end


%--------------------------------------------------------------------------
% PLOTS.
%--------------------------------------------------------------------------
if output_parms(3) > 0

    clf;

    axis square;

    if domain_type  ~= 0

        hold on;
        plot(nodes_x,nodes_y,'c.');
        plot(fek(:,1),fek(:,2),'ro');

        if size(tri,1) > 0
            for index=1:size(tri,1)
                plot(polygon_sides(tri(index,:),1),polygon_sides(...
                    tri(index,:),2),'g-');
            end
        end
        plot(polygon_sides(:,1),polygon_sides(:,2),'k-');
        hold off;

    else
        plot(nodes_x,nodes_y,'k.'); hold on;
    end

    hold off

    mx=min([polygon_sides(:,1); nodes_x]);
    MX=max([polygon_sides(:,1); nodes_x]);
    my=min([polygon_sides(:,2); nodes_y]);
    MY=max([polygon_sides(:,2); nodes_y]);

    % axis([mx,MX,my,MY])
end