
function [cub_pts, cub_weights, err]=polygon_cubature(polygon_vertices,...
    N,quad_rule)

%--------------------------------------------------------------------------
% OBJECT.
%----------
%
% THIS ROUTINE COMPUTES NODES "cub_pts" AND OF A CUBATURE RULE HAVING 
% "cub_weights". THE POLYGON IS SUBDIVIDED IN TRIANGLES BY A MINIMAL
% TRIANGULATION (see [1]) AND THEN, IN EACH TRIANGLE A CUBATURE FORMULA IS
% APPLIED (A TENSORIAL FORMULA IS USED HERE, BUT MODIFYING THE ATTACHED 
% ROUTINE ref_cubature_rule OTHER FORMULAE CAN BE CONSIDERED).
%
% PS. WE USED A BASIC TRIANGULATION ALGORITHM FOR THIS PURPOSE. A FASTER
% ONE HAS BEEN STUDIED BY CHAZELLE AND SEIDEL. OUR ALGORITHM WORKS IN 
% POLYGONS THAT ARE "SIMPLE", I.E. WITHOUT SELF-INTERSECTIONS.
%
%--------------------------------------------------------------------------
% INPUT.
%----------
%
% "polygon_vertices": VERTICES OF THE POLYGON (WITHOUT REPEATING THE 1ST 
%    INDEX AS LAST INDEX (TO CLOSE THE POLYGON)). IT IS AN "N x 2" MATRIX.
%
% "N": A CUBATURE RULE ON THE UNIT TRIANGLE (AKA SIMPLEX) IS NEEDED. THE
%    SIMPLEX IS SEEN AS A TRAPEZOIDAL REGION & A TENSORIAL FORMULA IS USED.
%    "N" IS THE NUMBER OF POINTS USED BY THIS RULE.
%
% "quad_rule" : IT CHOOSES THE QUADRATURE RULE USED BY THE TENSORIAL 
%    FORMULA DESCRIBED ABOVE.
%
%   [quad_rule=1]: TENSORIAL FEJER 1.
%   [quad_rule=2]: TENSORIAL FEJER 2.
%   [quad_rule=3]: TENSORIAL CLENSHAW CURTIS (VIA WEIGHTS).
%   [quad_rule=4]: TENSORIAL GAUSS-LEGENDRE.
%   [quad_rule=5]: TENSORIAL GAUSS-LEGENDRE-LOBATTO.
%
%--------------------------------------------------------------------------
% OUTPUT.
%----------
%
% "cub_pts","cub_weights": COLUMN VECTORS OF CUBATURE POINTS AND RESPECTIVE
%    WEIGHTS.
%
% "err": IF "err=1" THEN THE ROUTINE IS NOT BEEN COMPLETED AND A WARNING IS
%    WRITTEN ON THE DISPLAY.
%
%--------------------------------------------------------------------------
% FUNCTIONS USED IN THIS ROUTINE.
%---------------------------------
% 
% 1. minimal_triangulation
% 2. ref_cubature_rule 
% 3. triangle_map
% 4. find_ear (USED BY "minimal_triangulation")
% 5. polyarea (MATLAB BUILT_IN FUNCTION, USED BY "minimal_triangulation").
% 6. setdiff (MATLAB BUILT_IN FUNCTION, USED BY "minimal_triangulation").
% 7. verLessThan (MATLAB BUILT_IN FUNCTION, USED BY "find_ear").
% 8. inpolygon (MATLAB BUILT_IN FUNCTION, USED BY "find_ear").
% 9. cubature_trapezoidal_region (USED BY "minimal_triangulation").
%
% THE FUNCTION "ref_cubature_rule" IS ATTACHED TO THIS FILE.
%
% ALL THE OTHER FUNCTIONS ARE NOT ATTACHED TO THIS FILE.
%
% THE FUNCTION "cubature_trapezoidal_region" MAKES USE OF THE FUNCTION 
% "quadrature_rules_1D" (NOT ATTACHED TO "cubature_trapezoidal_region").
%
% THE FUNCTIONS "polyarea", "setdiff", "verLessThan", "inpolygon" ARE
% MATLAB BUILT-IN.
% 
%--------------------------------------------------------------------------
% TEST.
%-------
% 
% THIS ROUTINE HAS BEEN TESTED IN MATLAB 7.6.0.324 (R2008a).
% 
%--------------------------------------------------------------------------
% REFERENCES.
%-------------
%
% [1] Federica Basaglia, Un nuovo metodo di cubatura su poligoni (in
%     italian)
%
% [2] Chazelle and J. Incerpi, Triangulation and shape complexity, 
%     ACM Trans. on Graphics, vol. 3, pp. 135-152, 1984. 
%
% [3] B. Chazelle, Triangulating a simple polygon in linear time,  
%     Discrete Comput. Geom., vol. 6, pp. 485-524, 1991. 
%
% [4] David Eberly, Triangulation by Ear Clipping, 
%           http://www.geometrictools.com/ 
% 
% [5] R. Seidel, A simple and fast incremental randomized algorithm for  
%     computing trapezoidal decompositions and for triangulating polygons,  
%     Computational Geometry: Theory and Applications, vol. 1, no.1, 
%     pp. 51-64, 1991. 
%
% [6] Subhash Suri, Polygon Triangulation
%
% [7] http://cgm.cs.mcgill.ca/~godfried/teaching/cg-projects/97/ 
%            Ian/cutting_ears.html
% 
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Federica Basaglia
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 14, 2009
%--------------------------------------------------------------------------

% IF NO quad_rule IS SPECIFIED, GAUSS-LEGENDRE WILL BE USED TO COMPUTE A
% TENSORIAL RULE ON THE SIMPLEX.
if (nargin == 2)
    quad_rule =4;
end

% COMPUTE A MINIMAL TRIANGULATION. THE MATRIX tri HAS POSITIVE
% INTEGERS AS COMPONENTS.
[tri,err]=minimal_triangulation(polygon_vertices(:,1),polygon_vertices(:,2));
[ref_pts,ref_weights]=ref_cubature_rule(N,quad_rule);

cub_pts=[];
cub_weights=[];

%clf
%plot(polygon_vertices(:,1),polygon_vertices(:,2));

for index_triangulation=1:size(tri,1)
    
    % COMPUTING FOR ANY TRIANGLE A CUBATURE RULE. 
    % loc_triangle_pts IS THE LOCAL TRIANGLE THAT IS CONSIDERED.
    % A CUBATURE FORMULA IS COMPUTED ON THE UNIT TRIANGLE AND ADAPTED BY 
    % triangle_map AND polyarea TO THE LOCAL TRIANGLE.
    
    loc_tri_indices=tri(index_triangulation,:);
    loc_polygon_vertices=polygon_vertices(loc_tri_indices,:);
    loc_triangle_pts=triangle_map(loc_polygon_vertices,ref_pts);
    
    cub_pts=[cub_pts; loc_triangle_pts];
    triangle_area=polyarea(loc_polygon_vertices(:,1),...
        loc_polygon_vertices(:,2));
    loc_weights=2*ref_weights*triangle_area;
    cub_weights=[cub_weights; loc_weights];
    
    %hold on;
    %plot(cub_pts(:,1),cub_pts(:,2),'ko');
    %plot(loc_triangle_pts(:,1),loc_triangle_pts(:,2),'ro');
    %pause
    
end

%hold off;
%pause



%---------------------------------------
% ref_cubature_rule
%---------------------------------------

function [ref_pts,ref_weights]=ref_cubature_rule(N,quad_rule)

%--------------------------------------------------------------------------
% INPUT.
%----------
%
% "N": A CUBATURE RULE ON THE UNIT TRIANGLE (AKA SIMPLEX) IS NEEDED. THE
%    SIMPLEX IS SEEN AS A TRAPEZOIDAL REGION & A TENSORIAL FORMULA IS USED.
%    "N" IS THE NUMBER OF POINTS USED BY THIS RULE.
%
% "quad_rule" : IT CHOOSES THE QUADRATURE RULE USED BY THE TENSORIAL 
%    FORMULA DESCRIBED ABOVE.
%
%   [quad_rule=1]: TENSORIAL FEJER 1.
%   [quad_rule=2]: TENSORIAL FEJER 2.
%   [quad_rule=3]: TENSORIAL CLENSHAW CURTIS (VIA WEIGHTS).
%   [quad_rule=4]: TENSORIAL GAUSS-LEGENDRE.
%   [quad_rule=5]: TENSORIAL GAUSS-LEGENDRE-LOBATTO.
%
%--------------------------------------------------------------------------
% OUTPUT.
%----------
%
% "ref_pts", "ref_weights": COLUMN VECTORS OF CUBATURE POINTS AND RESPECTIVE
%    WEIGHTS.
%
%--------------------------------------------------------------------------

g1=inline('zeros(size(x))'); g2=inline('1-x');
[ref_pts,ref_weights]=cubature_trapezoidal_region(N,quad_rule,...
    0,1,g1,g2);



