
function V=vandermonde_2D(basis_type,pts,M,a,b,c,d)

%--------------------------------------------------------------------------
% OBJECT:
%---------
% THE MAIN PURPOSE OF THIS ROUTINE IS TO COMPUTE THE VANDERMONDE MATRIX OF
% DEGREE n WITH RESPECT TO A CERTAIN BASIS OF THE SPACE OF BIVARIATE
% POLYNOMIALS (IN SOME CURVILINEAR SPLINE CASES, THE BASIS IS SCALED ON THE
% RECTANGLE [a,b] x [c,d]).
%
% THE BASIS IS DETERMINED BY basis_type.
%--------------------------------------------------------------------------
% INPUTS:
%---------
%
% basis_type : DETERMINES THE BASIS.
%
%   [basis_type = 0] MONOMIAL (NOT SCALED).
%   [basis_type = 1] LEGENDRE (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                    CONTAINING THE DOMAIN).
%   [basis_type = 2] CHEBYSHEV (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                    CONTAINING THE DOMAIN).
%   [basis_type = 3] MONOMIAL (SCALED ON THE SMALLEST RECT. R=[a,b]x[c,d]
%                    CONTAINING THE DOMAIN).
%   [basis_type = 4] ORTHONORMAL DUBINER BASIS FOR UNIT TRIANGLE 
%                    [0 0; 1 0; 0 1].
%   [basis_type = 5] LOGAN SHEPP BASIS FOR UNIT DISC B(C=(0,0), r=1).
%   [basis_type = 6] SCALED MONOMIAL BASIS FOR THE INTERVAL.
%   [basis_type = 7] SCALED LEGENDRE BASIS FOR THE INTERVAL.
%   [basis_type = 8] SCALED CHEBYSHEV BASIS FOR THE INTERVAL.
%   [basis_type = 9] KOORNWINDER BASIS FOR UNIT DISC B(C=(0,0), r=1).
%   [basis_type = 10] DUBINER ORTHOGONAL BASIS FOR UNIT SIMPLEX W.R.T.
%                    THE EQUILIBRIUM MEASURE.
%   [basis_type =11] FAST CHEBYSHEV VANDERMONDE (SCALED ON THE SMALLEST 
%                    RECT. R=[a,b]x[c,d] CONTAINING THE DOMAIN).
%
% pts: POINTS IN WHICH THE VANDERMONDE MATRIX "V" MUST BE EVALUATED.
%
% M : DEGREE OF THE VANDERMONDE BASIS.
%
% a,b,c,d: THE BASIS IS SCALED FOR basis_type = 1, basis_type = 2, 
%    basis_type = 3, ON THE RECTANGLE [a,b] x [c,d]
% 
%---------- 
% OUTPUTS:
%----------
%
% V: VANDERMONDE MATRIX, WRT THE BASIS DETERMINED BY basis_type, EVALUATED
%    IN THE POINTS THAT ARE THE COMPONENTS OF THE VECTOR pts.
%
%--------------------------------------------------------------------------
% FUNCTIONS USED BY THIS ROUTINE.
%---------------------------------
% 
% 1. vandermonde_1D
% 2. chebvand2d
% 3. dubiner_simplex_equilibrium
% 4. koornwinder
% 5. logan_shepp
% 6. dubiner_legendre_weight
% 7. vandermonde_polygons
%
% NO FUNCTION IS ATTACHED HERE.
% 
% NOTE: dubiner_simplex_equilibrium USES THE EXTERNAL FUNCTION r_jacobi.
%
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 14, 2009
%--------------------------------------------------------------------------

switch basis_type
    
    % BOUNDED INTERVAL.
case 12
    % Here M is equal to 2n-1 in Gautschi's 
    % "On inverses of Vandermonde and confluent Vandermonde matrices II".
    Vintp=vandermonde_1D(3,pts,M,a,b);
    Vintp=Vintp';
    Vherm=[zeros(1,length(pts)); Vintp(size(Vintep,1)-1,:)]; 
    D=diag(0:size(Vherm,1)-1);
    Vherm=D*Vherm;
    
    V=[Vintp Vherm];
    
    % GENERAL BASIS ON A BOUNDED DOMAIN.
case 11
    V = chebvand2d(M,pts,[a b c d]);
    
    % DUBINER (EQUILIBRIUM) BASIS ON THE SIMPLEX.
case 10
    x=pts(:,1); y=pts(:,2);
    V=dubiner_simplex_equilibrium(M,x,y);
    
    % KOORNWINDER BASIS FOR UNIT DISK.
case 9
    x=pts(:,1);y=pts(:,2);
    V=koornwinder(M,x,y);
    
    % BOUNDED INTERVAL (SCALED CHEBYSHEV BASIS).
case 8
    pts=pts(:,1);
    V=vandermonde_1D(2,pts,M,a,b);
    
    % BOUNDED INTERVAL (SCALED LEGENDRE BASIS).
case 7
    pts=pts(:,1);
    V=vandermonde_1D(1,pts,M,a,b);
    
    % BOUNDED INTERVAL (SCALED MONOMIAL BASIS).
case 6
    pts=pts(:,1);
    V=vandermonde_1D(3,pts,M,a,b);
    
    % LOGAN-SHEPP FOR UNIT DISK.
case 5
    V=logan_shepp(pts,M);
    
    % DUBINER (BIVARIATE LEGENDRE WEIGHT) FOR SIMPLEX.
case 4
    V=dubiner_legendre_weight(pts,M);
        
    % BASIS FOR POLYGONS.
otherwise
    V=vandermonde_polygons(basis_type,pts,M,a,b,c,d);
    
end

