function V=vandermonde_polygons(basis_type,pts,polynomial_degree,a,b,c,d)

%--------------------------------------------------------------------------
% OBJECT.
%----------
%
% THE MAIN PURPOSE OF THIS ROUTINE IS TO COMPUTE THE VANDERMONDE MATRIX OF
% DEGREE n WITH RESPECT TO A CERTAIN BASIS OF THE SPACE OF BIVARIATE
% POLYNOMIALS ON A (CURVILINEAR) POLYGON DEFINED IN "define_polygon.m" BY 
% ITS VERTICES, IN THE COUNTERCLOCKWISE ORDER.
% THE BASIS IS DETERMINED BY basis_type AND MAY BE SCALED IN THE RECTANGLE 
% [a,b] x [c,d] THAT IN THE APPLICATIONS MAY BE THE SMALLEST RECTANGLE 
% WITH SIDES PARALLEL TO THE CARTESIAN AXIS, SAY R=[a,b]x[c,d], AND 
% CONTAINING THE DOMAIN.
%
%--------------------------------------------------------------------------
% INPUTS:
%---------
%
% basis_type: IT DETERMINES THE BASIS FOR THE DOMAIN.
%
%    [basis_type = 0] (TENSOR) MONOMIAL (NOT SCALED).
%    [basis_type = 1] (TENSOR) LEGENDRE, SCALED ON THE RECT. [a,b]x[c,d]
%                     THAT IN THE APPLICATIONS MAY BE THE SMALLEST RECT.
%                     R=[a,b]x[c,d ]CONTAINING THE DOMAIN.
%    [basis_type = 2] (TENSOR) CHEBYSHEV, SCALED ON THE RECT. [a,b]x[c,d]
%                     THAT IN THE APPLICATIONS MAY BE THE SMALLEST RECT.
%                     R=[a,b]x[c,d ]CONTAINING THE DOMAIN.
%    [basis_type = 3] (TENSOR) MONOMIAL, SCALED ON THE RECT. [a,b]x[c,d]
%                     THAT IN THE APPLICATIONS MAY BE THE SMALLEST RECT.
%                     R=[a,b]x[c,d ]CONTAINING THE DOMAIN
%
% pts : M x 2 MATRIX CONSISTING OF THE POINTS ON WHICH ALL THE POLYNOMIALS 
%       OF THE BASIS MUST BE EVALUATED.
%
% polynomial_degree : DEGREE OF THE VANDERMONDE BASIS.
%
% a,b,c,d: INTERVAL OF THE SPACE OF POLYNOMIALS OF DEGREE AT MOST n.
% 
%---------- 
% OUTPUTS:
%----------
%
% V: VANDERMONDE MATRIX V(pts,polynomial_degree), WRT THE BASIS DETERMINED 
%    BY basis_type, EVALUATED IN THE POINTS THAT ARE THE COMPONENTS OF THE 
%    MATRIX pts. IF {\phi_k} IS A BASIS, THEN 
%
%               V(:)=[\phi_1(:) \phi_2(:) ... \phi_N(:)]
%
%--------------------------------------------------------------------------
% FUNCTIONS USED BY THIS ROUTINE.
%---------------------------------
% 
% 1. vandermonde_1D
%
% THE FUNCTION vandermonde_1D IS EXTERNAL TO THIS ROUTINE.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 12, 2009
%--------------------------------------------------------------------------

V=[];

% STEP 1. ORDERING BASIS.

polynomial_indexes=[];
for k=1:(polynomial_degree+2)
    for j=1:((polynomial_degree+2)-k)
        polynomial_indexes_loc=[j-1 k-1];
        polynomial_indexes=[polynomial_indexes; polynomial_indexes_loc];
    end
end

% STEP 2. INITIALIZATION.

x=pts(:,1); y=pts(:,2);
elements_in_basis=size(polynomial_indexes,1);



% STEP 3. COMPUTING UNIVARIATE VANDERMONDE (x, y).
% THE BASIS IN THE MONOMIAL CASE IS OF THE FORM [1 x(1) x(1)^2; 1 x(2)
% x(2)^2].

if (basis_type > 0) | (basis_type) <= 3

else
    disp('WARNING:')
    disp('SETTING BASIS TYPE = 4 IN VANDERMONDE CUBATURE OVER POLYGONS.');
    basis_type=2;
end

Vx=vandermonde_1D(basis_type,x,polynomial_degree+1,a,b);
Vy=vandermonde_1D(basis_type,y,polynomial_degree+1,c,d);

% STEP 4. COMPUTING BIVARIATE BASIS.

index_j=polynomial_indexes(:,1);
index_k=polynomial_indexes(:,2);

V=Vx(:,index_j+1).*Vy(:,index_k+1);


